<?php
// +----------------------------------------------------------------+
// | Abook.php  													|
// +----------------------------------------------------------------+
// | Source-code (C) CalaCode.com 1998-2005 . All rights reserved	|
// | See http://calacode.com/license.ehtml for license agreement	|
// +----------------------------------------------------------------+
// | Date: Febuary 2005												|
// +----------------------------------------------------------------+

require_once('header.php');

require_once('Global.php');

class Abook
{

    var $db;

    // We want to ignore some fields that may exist if
	// the import file was produced from an @Mail abook export
	var $ignoreFields = array('id', 'DateAdded', 'UserType', 'EntryID', 'DateModified', 'Account');
	
		
	function Abook($arg)
	{
		global $atmail;

	    foreach ($arg as $k => $v)
	        $this->$k = $v;

	    if (!isset($atmail))
	    {
	        require_once('SQL.php');
	        require_once('Global.php');
            $this->db = new SQL();
            $this->db->table_names($this->Account);
            $this->Account_Escape = $this->db->quote($this->Account);
	    }
        else
        {
            $this->db = $atmail->share_db();
            $this->Account_Escape = $this->db->quote($atmail->Account);
        }

        $this->Type = "sql";

		$this->Abook = Filter::cleanSqlFieldNames($this->db->Abook); //'Abook_' . substr($atmail->Account, 0, 1);
		$this->Ugroup = $this->db->quote( $this->db->getvalue("SELECT Ugroup
															   FROM Users
															   WHERE Account = $this->Account_Escape") );

		// Find our domain-name
		$pos = strpos($atmail->Account, '@');
		$this->pop3host = $this->db->quote(substr($atmail->Account, $pos+1));

		// Default limit for abook entries, if undefined ( e.g from sync.php )
		$this->limit = '100000';

	}


	/**
	 * Return an array index
	 *
	 * @param string $type
	 * @param string $sort
	 *
	 * @return array $id
	 */
	function abookid($type, $sort = "UserEmail")
	{
		global $atmail;

		$id = array();

		$sort = Filter::cleanSqlFieldNames($sort);

		if ($type == "personal")
		{
			$query = "SELECT id FROM $this->Abook
					  WHERE Account = $this->Account_Escape
					  ORDER BY $sort DESC";

			$id = $this->db->sqlarray($query);
		}
		elseif ($type == "shared" && $pref['GlobalAbook'])
		{
			$query = "SELECT Abook_shared.id
					  FROM Abook_shared, AbookPermissions
					  WHERE Abook_shared.id = AbookPermissions.AbookID
					  AND (AbookPermissions.Account = ? OR AbookPermissions.Account = ?
					  	   OR AbookPermissions.Account = 'All Users' OR AbookPermissions.Account = ?)
					  ORDER BY $sort DESC";

			$data = array($this->Account_Escape, $atmail->Ugroup, $this->pop3host);
			$id = $this->db->sqlarray($query, $data);

		}
		elseif($pref['GlobalAbookRead'] || $pref['GlobalAbook'])
		{
			if ($sort == "UserEmail")
				$sort = "Account" ;

			$query = "SELECT id
					  FROM Users
					  WHERE Account = $this->Account_Escape
					  ORDER BY $sort";

			$id = $this->db->sqlarray($query);
		}

		return $id;
	}


	// Add a shared/global/group address-book entry
	function addshared($user)
	{
		global $atmail;

		// First, append the entry into the Abook_shared database
		$tmp = $this->Abook;
		$this->Abook = "Abook_shared";
		$abookid = $this->add($user);

		// Revert back to the original abook table
		$this->Abook = $tmp;

		$this->addsharedpermissions(array('WriteSelectedUsers' => $user['WriteSelectedUsers'], 'ReadSelectedUsers' => $user['ReadSelectedUsers'],
		'WriteSelectedGroups' => $user['WriteSelectedGroups'], 'ReadSelectedGroups' => $user['ReadSelectedGroups'], 'abookid' => $abookid) );

		# If called by the Outlook Sync utility, append the data into the lookup table
		if($user['EntryID'] && $user['DateModified'] && $abookid)
		$atmail->add_entryidmap($user['EntryID'], $abookid, "abook", $user['DateModified'] );

		return $abookid;
	}

	function addsharedpermissions($user)
	{
		global $atmail;

		// By default always add our account
		$this->addpermission($user['abookid'], $atmail->Account, '1');

		// Foreach read/write permission list, append into the database
		foreach( array('WriteSelectedUsers', 'ReadSelectedUsers', 'WriteSelectedGroups', 'ReadSelectedGroups') as $type)
		{
		    if (empty($user[$type]))
		        continue;

			// Toggle the permission setting
			$permission = ( strpos($type, 'Write') !== false ) ? "1" : "0";

			$ids = explode(",", $user[$type]);

			foreach($ids as $id)
				$this->addpermission($user['abookid'], $id, $permission);
		}

	}

	function addpermission($abookid, $account, $permission)
	{
		// Do not insert the entry if we are missing a field
		//if (!$account || !$abookid || !$permission);
		//    return 1

		// Check we do not already exist
		if($this->db->sqlgetfield("select AbookID from AbookPermissions where AbookID=? and Account=? and Permissions=?", array($abookid, $account, $permission)))
		return 1;

		// Append the data into the AbookPermissions table
		$res = $this->db->sqldo("INSERT INTO AbookPermissions ( AbookID, Account, Permissions ) VALUES ( ? , ?, ? ) ", array($abookid, $account, $permission));

		return 0;
	}

	// Add an addressbook entry
	function add($user)
	{
		// Take away ' , < and > characters from the Email ; they are illegal characters
		$user['Email'] = str_replace(array("'", '<', '>'), '', $user['Email']);
		$user['DateModified'] = ($user['DateModified'])? $user['DateModified'] : AtmailGlobal::outlook_date();
		$data = array(
			$user['UserEmail'],
			$user['UserEmail2'],
			$user['UserEmail3'],
			$user['UserEmail4'],
			$user['UserEmail5'],
			$user['UserFirstName'],
			$user['UserMiddleName'],
			$user['UserLastName'],
			$user['UserTitle'],
			$user['UserGender'],
			$user['UserDOB'],
			$user['UserHomeAddress'],
			$user['UserHomeCity'],
			$user['UserHomeState'],
			$user['UserHomeZip'],
			$user['UserHomeCountry'],
			$user['UserHomePhone'],
			$user['UserHomeMobile'],
			$user['UserHomeFax'],
			$user['UserURL'],
			$user['UserWorkCompany'],
			$user['UserWorkTitle'],
			$user['UserWorkDept'],
			$user['UserWorkOffice'],
			$user['UserWorkAddress'],
			$user['UserWorkCity'],
			$user['UserWorkState'],
			$user['UserWorkZip'],
			$user['UserWorkCountry'],
			$user['UserWorkPhone'],
			$user['UserWorkMobile'],
			$user['UserWorkFax'],
			$user['UserInfo'],
			$user['UserPgpKey'],
			$user['DateModified'],
			$user['EntryID']);

		$res = $this->db->sqldo( "INSERT INTO $this->Abook (
			UserEmail,
			UserEmail2,
			UserEmail3,
			UserEmail4,
			UserEmail5,
			UserFirstName,
			UserMiddleName,
			UserLastName,
			UserTitle,
			UserGender ,
			UserDOB,
			UserHomeAddress,
			UserHomeCity,
			UserHomeState,
			UserHomeZip,
			UserHomeCountry,
			UserHomePhone,
			UserHomeMobile,
			UserHomeFax,
			UserURL,
			UserWorkCompany,
			UserWorkTitle,
			UserWorkDept,
			UserWorkOffice,
			UserWorkAddress,
			UserWorkCity,
			UserWorkState,
			UserWorkZip,
			UserWorkCountry,
			UserWorkPhone,
			UserWorkMobile,
			UserWorkFax,
			UserInfo,
			UserPgpKey,
			Account,
			DateModified,
			EntryID,
			DateAdded
			) VALUES (
			?,?,?,?,?,?,
			?,?,?,?,?,?,
			?,?,?,?,?,?,
			?,?,?,?,?,?,
			?,?,?,?,?,?,
			?,?,?,?, $this->Account_Escape, ?, ?, {$this->db->NOW}
			)", $data);

		$id = $this->db->getid();

	    return $id;
	}

	/**
	 * Viel global contacts
	 *
	 * @param boolean [$sort]
	 * @param int [$start]
	 *
	 * @return array $h
	 */
	function viewglobal($sort = false, $start = 0)
	{
		global $pref, $atmail;

		if (!is_numeric($start))
            $start = 0;

		$h = array();

	    // Return empty array if the GlobalAbook setting is disabled
	    if ( !$pref['GlobalAbook'] && !$pref['GlobalAbookRead'] )
	    	return $h;

	    preg_match('/@(.*)/', $this->Account, $match);
	    $dom = $this->db->quote("%@$match[1]");

	    if ( $pref['GlobalAbook'] == 2 )
	    {
	    	$query = "SELECT Account
	    			  FROM Users
	    			  ORDER BY Account
	    			  LIMIT $start, $this->limit";
	    }
	    elseif ( $pref['GlobalAbook'] == 1 || $pref['GlobalAbookRead'])
	    {
	    	$query = "SELECT Account
	    			  FROM Users
	    			  WHERE Account LIKE $dom
	    			  ORDER BY Account
	    			  LIMIT $start, $this->limit";
	    }

	    $id = $this->db->sqlarray($query);

		if (!is_array($id))
			settype($id, 'Array');

	    foreach ($id as $user)
	    {
	        if ( !$user || $user == "@" )
	        	continue;

			if($sort == "sync")
			{

		        $db = $this->db->sqlhash("SELECT FirstName AS UserFirstName, LastName AS UserLastName,
		        						  TelHome AS UserHomePhone, FaxHome AS UserFaxHome, TelWork AS UserWorkPhone,
		        						  FaxWork AS UserWorkFax, TelMobile AS UserHomeMobile, Address AS UserHomeAddress,
		        						  City AS UserHomeCity, PostCode AS UserHomeZip, State AS UserHomeState,
		        						  Country AS UserHomeCountry, Account AS UserEmail, id
		        						  FROM Users
		        						  WHERE Account = ?", array($user));

				foreach($db as $id => $v)
				{
					$h[$db['id']]["$id"] = $v;
				}
					// If global addressbook, permissions are always read only, map the EntryID/Datemodified
					$h[$db['id']]["Permissions"] = "0";
					$h[$db['id']]["EntryID"] = $atmail->view_entryidmap($db['id'], "global");
					$h[$db['id']]["DateModified"] = $atmail->view_entryidmap($db['id'], "global", "1");

			} else {

		        $db = $this->db->sqlhash("SELECT FirstName, LastName, TelWork, TelHome, id
		        						  FROM Users
		        						  WHERE Account = ?", array($user));

				$h[$db['id']]['UserFullName'] = "{$db['FirstName']} {$db['LastName']}";
		        $h[$db['id']]['UserEmail']    = $user;
		        $h[$db['id']]['TelWork']      = $db['TelWork'];
		        $h[$db['id']]['TelHome']      = $db['TelHome'];

			}

			$h[$db['id']]['id']	= $db['id'];
	        $h[$db['id']]['UserGlobal']++;
	    }

	    return $h;
	}


	/**
	 * Calculate size of global addressbook
	 *
	 * @return int $result
	 */
	function get_global_abook_size()
	{
	    global $atmail, $pref;

	    if ( !$pref['GlobalAbook'] && !$pref['GlobalAbookRead'] )
	       return 0;

	    preg_match('/@(.*)/', $this->Account, $match);
	    $dom = $this->db->quote("%@$match[1]");

	    if ( $pref['GlobalAbook'] == 2 )
	    {
	    	$query = "SELECT COUNT(Account)
	    			  FROM Users";
	    }
	    elseif ( $pref['GlobalAbook'] == 1 || $pref['GlobalAbookRead'])
	    {
	    	$query = "SELECT COUNT(Account)
	    			  FROM Users
	    			  WHERE Account LIKE $dom";
	    }
	    $result = $this->db->getvalue($query);
	    return $result;

	}

	/**
	 * List all users in the addressbook
	 *
	 * @param string $sort sorking order
	 * @param string [$fields] needed fields
	 *
	 * @return array $h
	 */

	function view($sort, $fields='*', $start=0)
	{
	    global $atmail, $pref;

	    $h = array();

	    if ( !$sort ) $sort = "UserEmail" ;

	    $sort = Filter::cleanSqlFieldNames($sort);
	    $fields = Filter::cleanSqlFieldNames($fields);

	    if (!is_numeric($start))
	       $start = 0;

	    // List all users from the personal addressbook
	    $users = $this->db->sqlarray( "SELECT id
	                                   FROM $this->Abook
	                                   WHERE Account = $this->Account_Escape
	                                   ORDER BY $sort LIMIT $start,$this->limit");

	    if (is_array($users)) {

	        foreach ($users as $id)
	        {
	            $db = $this->db->sqlhash("SELECT $fields
	                                      FROM $this->Abook
	                                      WHERE Account = $this->Account_Escape AND id = ?", array($id));

	            $indexid = $db['id'];

	            foreach ($db as $k => $v)
	            $h[$indexid][$k] = $v;

	            // Disallow the user to edit the entry if 'Read' access is granted to the user via the Webadmin Group
	            if ($pref['GlobalAbookRead'] && !$pref['GlobalAbook'])
	            $h[$indexid]['Permissions'] = 0 ;

	        }
	    }

	    return $h;
	}

	function get_abook_size()
	{
	    global $atmail;
	    return $this->db->getvalue("SELECT COUNT(id)
	                                FROM $this->Abook
	                                WHERE Account = $this->Account_Escape");
	}

	/**
	 * Load shared contacts
	 *
	 * @param string $sort sorting order
	 * @param string $fields
	 *
	 * @return array $h
	 */
	function viewshared($sort, $fields = '', $start = 0)
	{
	    global $atmail, $pref;

	    if (!is_numeric($start))
            $start = 0;

	    $sort = Filter::cleanSqlFieldNames($sort);
        $fields = Filter::cleanSqlFieldNames($fields);

	    $h = array();

	    // Next, loop through any shared entrys the user has permission over
	    $users = $this->db->sqlarray( "SELECT Abook_shared.id
	                                   FROM Abook_shared, AbookPermissions
	                                   WHERE Abook_shared.id = AbookPermissions.AbookID
	                                   AND ( AbookPermissions.Account = $this->Account_Escape
	                                         OR AbookPermissions.Account = $this->Ugroup
	                                         OR AbookPermissions.Account = 'All Users'
	                                         OR AbookPermissions.Account = $this->pop3host)
	                                   GROUP BY UserEmail ORDER
	                                   BY $sort limit $start,$this->limit");

	    foreach ($users as $id)
	    {
	        $id = $this->db->quote($id);

	        if (strlen($fields) > 0)
	        {
	            $db = $this->db->sqlhash("SELECT $fields
				                          FROM Abook_shared, AbookPermissions
				                          WHERE Abook_shared.id = $id AND AbookPermissions.AbookID = $id
				                          AND ( AbookPermissions.Account = $this->Account_Escape
				                                OR AbookPermissions.Account = $this->Ugroup
				                                OR AbookPermissions.Account = 'All Users'
				                                OR AbookPermissions.Account = $this->pop3host)");
	        }
	        else
	        {
	            $db = $this->db->sqlhash("SELECT Abook_shared.*, AbookPermissions.Permissions
				                          FROM Abook_shared, AbookPermissions
				                          WHERE Abook_shared.id = $id AND AbookPermissions.AbookID = $id
				                          AND ( AbookPermissions.Account = $this->Account_Escape
				                                OR AbookPermissions.Account = $this->Ugroup
				                                OR AbookPermissions.Account = 'All Users'
				                                OR AbookPermissions.Account = $this->pop3host)
				                          ORDER BY AbookPermissions.Permissions
				                          DESC LIMIT 1");
	        }

	        $indexid = $db['id'];

	        foreach ($db as $k => $v) {
	            $h[$indexid][$k] = $v;
	        }

	        $h[$indexid]['Shared'] = 1;

	        // Disallow the user to edit the entry if 'Read' access is granted to the user via the Webadmin Group
	        if ($pref['GlobalAbookRead'] && !$pref['GlobalAbook'])
	        {
	            $h[$indexid]['Permissions'] = 0 ;
	        }
	    }

	    return $h;
	}


	/**
	 * Calculate size of shared addressbook
	 *
	 * @return int $result
	 */
	function get_shared_abook_size()
	{
	    global $atmail;

	    $result = 0;
	    $query = "SELECT count(Abook_shared.id)
				  FROM Abook_shared, AbookPermissions
				  WHERE Abook_shared.id = AbookPermissions.AbookID
				  AND (AbookPermissions.Account = $this->Account_Escape
	                   OR AbookPermissions.Account = $this->Ugroup
	                   OR AbookPermissions.Account = 'All Users'
	                   OR AbookPermissions.Account = $this->pop3host)
	              GROUP BY UserEmail";

	    $result = $this->db->getvalue($query);
	    return $result;
	}


	/**
	 * List all groups in the personal addressbook
	 *
	 * @return array $h selected groups
	 */
	function listgroup()
	{
		global $atmail;

	    $h = array();

	    $abookGroup = Filter::cleanSqlFieldNames($this->db->AbookGroup);

	    // users from addressbooks for current account
	    $users = $this->db->sqlarray("SELECT id
	                                  FROM $abookGroup
	                                  WHERE Account = $this->Account_Escape AND GroupEmail != ''");

	    foreach ($users as $user)
	    {
	        $user = $this->db->quote($user);
	        $v    = $this->db->getvalue("SELECT GroupName
	                                     FROM $abookGroup
	                                     WHERE id = $user");

	        $h[$v] .= $this->db->getvalue("SELECT GroupEmail
	                                       FROM $abookGroup
	                                       WHERE id = $user") . ", ";
	    }

	    return $h;
	}

	/**
	 * List all groups in the shared addressbook
	 *
	 * @param int $id shared group id
	 * @return array $db
	 */
	function listsharedgroup($id)
	{
		global $atmail, $pref;

		// Disable function if global addressbook disabled
		if (!$pref['GlobalAbook'])
            return;

	    $h = array();

		$id = $this->db->quote($id);

		// Load all the data for the shared group, if our account has permissions to do so
		$db = $this->db->sqlhash( "SELECT Abook_shared.*, AbookPermissions.Permissions
		                             FROM Abook_shared, AbookPermissions
		                             WHERE Abook_shared.id=$id AND AbookPermissions.AbookID=$id
		                             AND (AbookPermissions.Account=$this->Account_Escape
		                                  OR AbookPermissions.Account=$this->Ugroup
		                                  OR AbookPermissions.Account='All Users'
		                                  OR AbookPermissions.Account=$this->pop3host)
		                             AND ( AbookPermissions.Permissions='1' OR AbookPermissions.Permissions='0') order by Permissions desc");

		// Return if no results found
		if (!$db['id'])
			return;

		// List users within the group
		$db['UsersArray'] = join(",", $this->db->sqlarray("SELECT AbookGroup_shared.GroupEmail
														   FROM Abook_shared, AbookGroup_shared
														   WHERE Abook_shared.id = $id
														   AND AbookGroup_shared.GroupName = Abook_shared.UserEmail"));
														   //AND AbookGroup_shared.Account = $this->Account_Escape") );

		// Next, select which users have read/write access over the entry
		$ids = $this->db->sqlarray("SELECT Account
									FROM AbookPermissions
									WHERE AbookID = $id");

		// Select which users have permission to edit the entry
		list($db['PermissionsArray'], $db['GroupPermissions']) = $atmail->generatepermissions($id, "AbookPermissions", "AbookID", $ids);

	    return $db;
	}

	/**
	 * List a selected group in the personal addressbook
	 *
	 * @param int $id personal group id
	 * @return array $db
	 */
	function listpersonalgroup($id)
	{
		global $atmail;

	    $h = array();

		$id = $this->db->quote($id);
        $abookGroup = Filter::cleanSqlFieldNames($this->db->AbookGroup);

		// Load all the data for the shared group, if our account has permissions to do so
		$db = $this->db->sqlhash("SELECT *
								  FROM $this->Abook
								  WHERE $this->Abook.id = $id AND $this->Abook.Account = $this->Account_Escape");

		// List users within the group
		$users = $this->db->sqlarray("SELECT $abookGroup.GroupEmail
									  FROM $this->Abook, $abookGroup
									  WHERE $this->Abook.id = $id  AND $abookGroup.GroupName = $this->Abook.UserEmail
									  AND $abookGroup.Account = $this->Account_Escape");

		$db['UsersArray'] = implode(",", $users);

	    return $db;
	}

	// Search the user table
	function search($sort=null, $db=null, $start=0)
	{
		if ($db['abookview'] == 'ldap')
		{
			return $this->_search_ldap($db);
		}

		global $atmail, $pref;

		// Init some vars
	    $h = $users = $results = array();
        $extend = '';

        if (!is_numeric($start))
            $start = 0;

	    if (empty($sort))
	    	$sort = "Account" ;
        else
    	    $sort = Filter::cleanSqlFieldNames($sort);

    	$db['SearchType'] = Filter::stringMatch($db['SearchType'], array('and', 'or'));

		if (!$db['SearchType'])
			$db['SearchType'] = 'and';

		if ($db['SearchType'] == 'or')
			$extend = 'and (';

		foreach ( array('FirstName', 'LastName', 'Account', 'UserWorkCompany', 'UserHomeAddress', 'UserHomeCity', 'UserHomeState', 'UserHomeCountry') as $field )
		{
			// If the user is searching a global or personal addressbook, make the search query ( optimize and only search fields required )
			if ($db['abookview'] == "personal" || $db['abookview'] == "shared")
			{
				// Toggle which database we are using
				if ($db['abookview'] == "personal")
					$type = $this->Abook;
				else
					$type = "Abook_shared";

				if ($field == "FirstName" && !empty($db[$field]) )
					$extend .= "{$db['SearchType']} $type.UserFirstName LIKE " . $this->db->quote("%{$db[$field]}%") . " ";

				elseif ($field == "LastName" && !empty($db[$field]) )
					$extend .= "{$db['SearchType']} $type.UserLastName LIKE " . $this->db->quote("%{$db[$field]}%") . " ";

				elseif ($field == "Account" && !empty($db[$field]) )
					$extend .= "{$db['SearchType']} $type.UserEmail LIKE " . $this->db->quote("%{$db[$field]}%") . " ";

				// Other fields are correctly formatted with the field-name
				elseif (!empty($db[$field]))
					$extend .= "AND $type.$field LIKE " . $this->db->quote("%{$db[$field]}%") . " ";
			}
			else
			{
				// Search the user system database for the contact information

				if ($field == "UserHomeCity" && $db[$field])
					$extend .= "{$db['SearchType']} City LIKE " . $this->db->quote("%$db[$field]%") . " ";

				elseif($field == "UserHomeState" && $db[$field])
					$extend .= "{$db['SearchType']} State LIKE " . $this->db->quote("%$db[$field]%") . " ";

				elseif ($field == "UserHomeCountry" && $db[$field])
					$extend .= "{$db['SearchType']} Country LIKE " . $this->db->quote("%$db[$field]%") . " ";

				elseif ($field == "UserHomeAddress" && $db[$field])
					$extend .= "{$db['SearchType']} Address LIKE " . $this->db->quote("%$db[$field]%") . " ";

				// Other fields are correctly formatted with the field-name, but the Company field is unavailable for the Users
				elseif ($db[$field] && $field != "UserWorkCompany")
					$extend .= "{$db['SearchType']} $field LIKE " . $this->db->quote("%$db[$field]%") . " ";
			}

			$id = $db['id'] ? $db['id'] : 0;

	        if ($id > 0 && is_numeric($id)) {
                $extend .= " AND $type.id > $id ";
			}

			//if ($atmail->isset_chk($db['limit']) && is_numeric($db['limit']))
			//	$limit = "LIMIT {$db['limit']}";
			//else
			//	$limit = "LIMIT 300";

			$limit = "LIMIT $start,$this->limit";

			$db[$field] = $this->db->quote("%$db[$field]%") ;
		}

		if ($db['abookview'] == "personal")
		{
			// If there is no search query, specify a default searching for valid email-accounts
			if (!$extend)
			$extend = " AND UserEmail LIKE '%@%'" ;

			elseif ($db['SearchType'] == 'or')
			{
				$extend = str_replace('and (or', 'and (', $extend);
				$extend .= ')';
			}

			$results['FullTotal'] = $this->db->sqlgetfield("SELECT COUNT(id)
															FROM $this->Abook
															WHERE Account = $this->Account_Escape $extend
															ORDER BY $this->Abook.$sort");

			// Search the users Personal addressbook only
			$users = $this->db->sqlarray("SELECT id
										  FROM $this->Abook
										  WHERE Account = $this->Account_Escape $extend
										  ORDER BY $this->Abook.$sort $limit");
		}
		elseif ($db['abookview'] == "shared" && $pref['GlobalAbook'])
		{
			if ($db['SearchType'] == 'or')
			{
				$extend = str_replace('and (or', 'and (', $extend);
				$extend .= ')';
			}

			$results['FullTotal'] = $this->db->sqlgetfield("SELECT COUNT(Abook_shared.id)
															FROM Abook_shared,AbookPermissions
															WHERE Abook_shared.id=AbookPermissions.AbookID
															AND ( AbookPermissions.Account = $this->Account_Escape
															OR AbookPermissions.Account = $this->Ugroup
															OR AbookPermissions.Account= 'All Users'
															OR AbookPermissions.Account = $this->pop3host)
															$extend
															GROUP BY Abook_shared.UserEmail
															ORDER BY Abook_shared.$sort");

				// Search the shared addressbook, for only accounts the user has permission over ( user or group wide )
				$users = $this->db->sqlarray( "SELECT Abook_shared.id AS id
											   FROM Abook_shared, AbookPermissions
		    								   WHERE Abook_shared.id = AbookPermissions.AbookID
		    								   AND (AbookPermissions.Account = $this->Account_Escape
		    								   OR AbookPermissions.Account = $this->Ugroup
		    								   OR AbookPermissions.Account = 'All Users'
		    								   OR AbookPermissions.Account = $this->pop3host)
		    								   $extend
		    								   ORDER BY Abook_shared.$sort $limit");
		}
		elseif ($pref['GlobalAbook'] || $pref['GlobalAbookRead'])
		{
			if ($sort == "UserEmail")
				$sort = "Account" ;

			if ($db['SearchType'] == 'or')
			{
				$extend = str_replace('and (or', 'and (', $extend);
				$extend .= ')';
			}

			if ($pref['GlobalAbook'] == "1")
			{
				preg_match('/@(.*)/', $atmail->Account, $match);
				$extend .= "AND Account LIKE '%@$match[1]'";
			}
			elseif ($pref['GlobalAbook'] == "2")
				$extend .= " AND Account LIKE '%@%'";

			// Otherwise search the system-wide addressbook
			$results['FullTotal'] = $this->db->sqlgetfield("SELECT count(id)
															FROM Users
															WHERE Account IS NOT NULL $extend
															ORDER BY $sort");

		    $users = $this->db->sqlarray("SELECT id
		    							  FROM Users
		    							  WHERE Account IS NOT NULL $extend
		    							  ORDER BY $sort $limit");
		} else {
			catcherror('Unknown search supplied');
		}

	    foreach ($users as $id)
	    {
	        $id = stripslashes($id);

			$id_escape = $this->db->quote($id);

			if ($db['abookview'] == "personal")
			{
			    $h = $this->db->sqlhash( "SELECT UserEmail, UserFirstName, UserLastName, UserGender, id,
			    						  UserWorkPhone, UserHomePhone
			    						  FROM $this->Abook
			    						  WHERE id = $id_escape");

				// The UserEmail is the unique identifier for the personal addressbook
				$h['Account'] = $h['UserEmail'];

			}
			elseif ($db['abookview'] == "shared" && $pref['GlobalAbook'])
			{
			    $h = $this->db->sqlhash( "SELECT Abook_shared.UserEmail, Abook_shared.UserFirstName, Abook_shared.UserLastName,
			    						  Abook_shared.UserGender, Abook_shared.id, Abook_shared.UserWorkPhone,
			    						  Abook_shared.UserHomePhone,AbookPermissions.Permissions
			    						  FROM Abook_shared, AbookPermissions
			    						  WHERE Abook_shared.id = $id_escape AND AbookPermissions.AbookID = $id_escape");

				// The UserEmail is the unique identifier for the global addressbook
				$h['Account'] = $h['UserEmail'];
			}
			elseif ($pref['GlobalAbook'] || $pref['GlobalAbookRead'])
			{
		        $h = $this->db->sqlhash("SELECT Account, FirstName, LastName
		        						 FROM Users
		        						 WHERE id = $id_escape ");

				$h['UserEmail'] = $h['Account'];
				$h['FullName'] = "{$h['FirstName']} {$h['LastName']}";
				$h['UserFirstName'] = $h['FirstName'];
				$h['UserLastName'] = $h['LastName'];
			}

	        if ( !$h['Account'] )
	        	continue;

	        foreach ( $h as $k => $v)
			{
				if ($pref['iconv'] && defined('ICONV_VERSION'))
					$results[$id][$k] = iconv("utf-8", "utf-8", $v);
				else
					$results[$id][$k] = $v;
			}

			// Escape ' in users First/Last name - Otherwise breaks the JS function
			$results[$id]['FirstName'] = addslashes($results[$id]['FirstName']);
			$results[$id]['LastName'] = addslashes($results[$id]['LastName']);

	    }

	    return $results;
	}

	//
	function addgroup($group, $email)
	{
		global $atmail;

		$groupTable = Filter::cleanSqlFieldNames($this->db->AbookGroup);
		$res = $this->db->sqldo("INSERT INTO $groupTable (GroupName, GroupEmail, Account) VALUES (?,?,?)", array($group, $email, $this->Account));

	    return;
	}

	// List all users in the addressbook
	function addgroupshared($group, $email)
	{
		global $atmail;

		$this->db->sqldo("INSERT INTO AbookGroup_shared (GroupName, GroupEmail, Account) VALUES (?,?,?)", array($group, $email, $this->Account));

	    return;
	}

	/**
	 * List all users in the addressbook
	 *
	 * @param string $sort sorting order
	 * @return array
	 */
	function viewgroup($sort)
	{

		global $atmail;

		$h = array();

		$abookGroup = Filter::cleanSqlFieldNames($this->db->AbookGroup);

		$users = $this->db->sqlarray("SELECT GroupName
		                                FROM $abookGroup
		                                WHERE Account = $this->Account_Escape
		                                GROUP BY GroupName");

	    foreach ($users as $group)
	    {
			$group_escape = $this->db->quote($group);

	        $email = $this->db->sqlarray("SELECT GroupEmail
	                                        FROM $abookGroup
	                                        WHERE Account = $this->Account_Escape
	                                        AND GroupName = $group_escape AND GroupEmail IS NOT NULL");

	        foreach ($email as $v)
	        {
	        	 if ( !$v )
	        	 	continue;
	        	 $h[$group][$v]++;
	        }

	    }

	    return $h;
	}

	/**
	 * List all groups in the shared addressbook
	 *
	 * @param string $sort sorting order
	 * @return array $h shared groups
	 */
	function viewsharedgroup($sort)
	{
		global $atmail;

	    $h = array();

		$users = $this->db->sqlarray("SELECT AbookGroup_shared.GroupName
		                                FROM AbookGroup_shared, Abook_shared, AbookPermissions
		                                WHERE Abook_shared.UserEmail = AbookGroup_shared.GroupName
		                                AND AbookPermissions.AbookID = Abook_shared.id
		                                AND ( AbookPermissions.Account = $this->Account_Escape
		                                      OR AbookPermissions.Account = $this->Ugroup
		                                      OR AbookPermissions.Account = 'All Users'
		                                      OR AbookPermissions.Account = $this->pop3host)
		                                GROUP BY GroupName");

	    foreach ($users as $group)
	    {
			$group_escape = $this->db->quote($group);

	        $email = $this->db->sqlarray("SELECT GroupEmail
	                                        FROM AbookGroup_shared
	                                        WHERE GroupName = $group_escape AND GroupEmail IS NOT NULL");

	        foreach ($email as $v)
	        {
	        	if ( !$v )
	        		continue;
	        	$h[$group][$v]++;
	        }

			$h[$group]["Shared"] = 1;

		}

	    return $h;
	}

	// Return an array of User email-addresses
	function getgroup($group)
	{
		global $atmail;

		$group = $this->db->quote($group);
		$abookGroup = Filter::cleanSqlFieldNames($this->db->AbookGroup);

		$users = $this->db->sqlarray("select GroupEmail from $abookGroup where Account=$this->Account_Escape and GroupName=$group group by GroupEmail");

		$allusers = array();

		foreach($users as $user)
		{

			// Skip a bad record
			if(empty($user))
				continue;

			if (strpos($user, '@') === false)
				array_push($allusers, $this->getgroup($user));
			else
				array_push($allusers, $user);

		}

		return $allusers;
	}

	// Return an array of User email-addresses from the shared addressbook
	function getsharedgroup($group)
	{
		global $atmail;

		$group = $this->db->quote($group);

		$users = $this->db->sqlarray("SELECT AbookGroup_shared.GroupEmail
									  FROM AbookGroup_shared, Abook_shared, AbookPermissions
									  WHERE Abook_shared.UserEmail = AbookGroup_shared.GroupName
									  AND AbookPermissions.AbookID=Abook_shared.id AND AbookGroup_shared.GroupName = $group
									  AND (AbookPermissions.Account = $this->Account_Escape OR AbookPermissions.Account = $this->Ugroup
									  	   OR AbookPermissions.Account = 'All Users' OR AbookPermissions.Account=$this->pop3host)
									 GROUP BY GroupEmail");

		$allusers = array();

		foreach($users as $user)
		{
			// Skip a bad record
			if(empty($user))
				continue;

			if (strpos($user, '@') === false)
				array_push($allusers, $this->getsharedgroup($user));
			else
				array_push($allusers, $user);

		}

		return $allusers;
	}

	/**
	 * View a specified user
	 *
	 * @param string $user
	 * @param string $type type of selected user
	 * @return array
	 */
	function viewuser($user, $type)
	{
		global $atmail, $pref;

		$db = array();

		$user = $this->db->quote($user);

		// If a global entry dump the record from the database
	    if ($type == "global" && ($pref['GlobalAbook'] || $pref['GlobalAbookRead']))
	    {
	        $db = $this->db->sqlhash("SELECT *
	        						  FROM Users
	        						  WHERE id = $user");

			// Reformat the user fields to reflect the addresbook data structure
	        $db['UserEmail']      = $db['Account'];
	        $db['UserFullName']   = "{$db['FirstName']} {$db['LastName']}";
	        $db['UserAddress']    = $db['Address'];
	        $db['UserCity']       = $db['City'];
	        $db['UserState']      = $db['State'];
	        $db['UserZip']        = $db['PostCode'];
	        $db['UserCountry']    = $db['Country'];
	        $db['UserIndustry']   = $db['Industry'];
	        $db['UserOccupation'] = $db['Occupation'];
	        $db['UserOtherEmail'] = $db['OtherEmail'];
			$db['DateAdded']      = $db['DateCreate'];

			// Pase the date into a more readable format
			$db['DateAdded'] = preg_replace('/(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/', '$3/$2/$1 $4:$5:$6', $db['DateAdded']);

	    }
	    elseif ($type == "shared" && $pref['GlobalAbook'])
	    {
			// If a shared contact, query the Abook_shared table
			$db = $this->db->sqlhash("SELECT DISTINCT Abook_shared.*, AbookPermissions.Permissions
									  FROM Abook_shared, AbookPermissions
									  WHERE Abook_shared.id = $user AND AbookPermissions.AbookID = $user
									  AND (AbookPermissions.Account = $this->Account_Escape OR AbookPermissions.Account = $this->Ugroup
										   OR AbookPermissions.Account='All Users' OR AbookPermissions.Account=$this->pop3host)
								      AND (AbookPermissions.Permissions = '1' OR AbookPermissions.Permissions = '0')
									  ORDER BY AbookPermissions.Permissions DESC
									  LIMIT 1 ");

			// Disallow the user to edit the entry if 'Read' access is granted to the user via the Webadmin Group
			if ($pref['GlobalAbookRead'] && !$pref['GlobalAbook'])
				$db['Permissions'] = 0 ;

			// Next, select which users have read/write access over the entry
			$id = $this->db->sqlarray("SELECT Account
									   FROM AbookPermissions
									   WHERE AbookID = $user");

			// Load the permissions array for the select shared entry
			list($db['PermissionsArray'], $db['GroupPermissions']) = $atmail->generatepermissions($user, "AbookPermissions", "AbookID", $id);

			$usergroup = $this->db->sqlarray("SELECT GroupName
											  FROM AbookGroup_shared
											  WHERE Account = $this->Account_Escape AND GroupEmail=?
											  GROUP BY GroupName", $db['UserEmail'] );

			if (is_array($usergroup)) {
				foreach ($usergroup as $ug) {
					if(!$ug)
						continue;
					$db['UserGroups'] .= "UserGroups[\"$ug\"] = 1;\n";
				}
			}
		}
		else
		{
			// Otherwise the contact is a 'personal' addressbook entry. Permission is always granted for read/write access
	        $db = $this->db->sqlhash("SELECT *
	        						  FROM $this->Abook
	        						  WHERE Account = $this->Account_Escape AND id = $user" );

			$db['Permissions'] = 1;
			$UserDOB = explode(" ", $db['UserDOB']);
            $db['UserDOB'] = $UserDOB[0];

            $abookGroup = Filter::cleanSqlFieldNames($this->db->AbookGroup);

			$usergroup = $this->db->sqlarray("SELECT GroupName
											  FROM $abookGroup
											  WHERE Account = ? AND GroupEmail = ?
											  GROUP BY GroupName", array($this->Account_Escape, $db['UserEmail']));


			foreach ($usergroup as $ug)
			{
				if (!$ug)
					continue;
				$db['UserGroups'] .= "UserGroups[\"$ug\"] = 1;\n";
			}
	    }

		$db['UserInfo'] = str_replace("\n", "<BR>", $db['UserInfo']);

	    return $db;
	}

	/**
	 * Delete the specified addressbook item.
	 *
	 * @param string $user UserEmail
	 * @param int $id user id
	 */
	function delete($user, $id)
	{
		global $atmail;

		if (strlen($user) > 0)
		{
			$this->db->sqldo("DELETE
			                  FROM $this->Abook
			                  WHERE UserEmail = ? AND id = ? AND Account = $this->Account_Escape", array($user, $id));
		}
		else
		{
			$this->db->sqldo("DELETE
			                  FROM $this->Abook
			                  WHERE id = ? AND Account = $this->Account_Escape", $id);
		}

	}

	/**
	 * Delete all the entrys in the database with
	 * the Groupname, for our Account.
	 *
	 * @param string $group selected group name
	 */
	function deletegroup($group)
	{
		global $atmail;

		$group = $this->db->quote($group);
        $abookGroup = Filter::cleanSqlFieldNames($this->db->AbookGroup);

		// Delete the reference from the AbookGroup, and Personal Abook tables
	    $this->db->sqldo("DELETE
	                      FROM $abookGroup
	                      WHERE GroupName = $group AND Account = $this->Account_Escape" );

	    // Delete group name from addressbook table
	    $this->db->sqldo("DELETE
	                      FROM $this->Abook
	                      WHERE UserEmail= $group AND Account = $this->Account_Escape" );

	}

	/**
	 * Delete the specified addressbook item for shared contacts.
	 *
	 * @param string $user
	 * @param int $id
	 */
	function deleteshared($user, $id)
	{
		global $atmail;

		$user = $this->db->quote($user);
		
		// First, check our account has permissions to delete shared addressbook entry
		$permission = $this->check_permissions($id, $user);

		$id = $this->db->quote($id);
		
		if ($permission)
		{
	    	$this->db->sqldo( "DELETE
	    					   FROM Abook_shared
	    					   WHERE UserEmail = $user AND id = $id" );

	    	$this->db->sqldo( "DELETE
	    					   FROM AbookPermissions
	    					   WHERE AbookID = $id");
		}

	    return;
	}

	/**
	 * Delete the specified addressbook item for shared contacts.
	 *
	 * @param string $user
	 * @param int $id
	 */
	function deletesharedgroup($user, $id)
	{
		global $atmail;

		$user = $this->db->quote($user);
		
		// First, check our account has permissions to delete shared addressbook entry
		$permission = $this->check_permissions($id, $user);

		$id = $this->db->quote($id);

		// Delete selected user
		if ($permission)
		{
	    	$this->db->sqldo( "DELETE
	    					   FROM Abook_shared
	    					   WHERE id = $id" );

	    	$this->db->sqldo( "DELETE
	    					   FROM AbookPermissions
	    					   WHERE AbookID = $id");

	    	$this->db->sqldo( "DELETE
	    					   FROM AbookGroup_shared
	    					   WHERE GroupName = $user");
		}

	    return;
	}

	function updatefield($user)
	{
		global $atmail;

		$args = array();
		$sql = '';

		foreach($user as $k => $v)
		{
		    $k = Filter::cleanSqlFieldNames($k);

			if ($v)
			{
				array_push($args, $v);
				if ($k == "id" || $k == 'Shared') continue;

				if ($v == "(null)")
					$sql .= "$k = " . $this->db->quote('') . ", ";
				else
					$sql .= "$k = " . $this->db->quote($v) . ", ";
			}
		}

		$sql = preg_replace('/, $/', '', $sql);

		// Detect if we update a shared or local addressbook
		if (!$user['Shared'] )	{
            $sql = "update $this->Abook set $sql where id = ? and Account=$this->Account_Escape";
		} elseif ( $this->check_permissions($user['id']))	{
            $sql = "update Abook_shared set $sql where id = ?";
		} else	{
    		// Otherwise, if we have no permissions, just update the lookup table
    		$atmail->add_entryidmap($user['EntryID'], $user['id'], "abook", $user['DateModified']);

    		return 1;
		}

		// Execute the SQL Query
		$this->db->sqldo($sql, $user['id']);

		if ($user['Shared']) {
    		// Last, update the SQL table for the SharedLookup table. This will automatically refresh data
    		// sync'd with Outlook/WebMail
    		$this->update_sharedlookup($user['id'], "abook", $user['DateModified'] );
		}

		return;
	}

	function update($user)
	{
		global $atmail;

		$user['id'] = $this->db->quote($user['id']);

		$user['DateModified'] = $atmail->outlook_date();

	    // Update a users addressbook data
	    $this->db->sqldo( "update $this->Abook set
			UserEmail = ?,
			UserEmail2 = ?,
			UserEmail3 = ?,
			UserEmail4 = ?,
			UserEmail5 = ?,
			UserFirstName = ?,
			UserMiddleName = ?,
			UserLastName = ?,
			UserTitle = ?,
			UserGender = ?,
			UserDOB = ?,
			UserHomeAddress = ?,
			UserHomeCity = ?,
			UserHomeState = ?,
			UserHomeZip = ?,
			UserHomeCountry = ?,
			UserHomePhone = ?,
			UserHomeMobile = ?,
			UserHomeFax = ?,
			UserURL = ?,
			UserWorkCompany = ?,
			UserWorkTitle = ?,
			UserWorkDept = ?,
			UserWorkOffice = ?,
			UserWorkAddress = ?,
			UserWorkCity = ?,
			UserWorkState = ?,
			UserWorkZip = ?,
			UserWorkCountry = ?,
			UserWorkPhone = ?,
			UserWorkMobile = ?,
			UserWorkFax = ?,
			UserInfo = ?,
			UserPgpKey = ?,
			DateModified = ?
			where Account = $this->Account_Escape and id={$user['id']}",

			array(
			$user['UserEmail'],
			$user['UserEmail2'],
			$user['UserEmail3'],
			$user['UserEmail4'],
			$user['UserEmail5'],
			$user['UserFirstName'],
			$user['UserMiddleName'],
			$user['UserLastName'],
			$user['UserTitle'],
			$user['UserGender'],
			$user['UserDOB'],
			$user['UserHomeAddress'],
			$user['UserHomeCity'],
			$user['UserHomeState'],
			$user['UserHomeZip'],
			$user['UserHomeCountry'],
			$user['UserHomePhone'],
			$user['UserHomeMobile'],
			$user['UserHomeFax'],
			$user['UserURL'],
			$user['UserWorkCompany'],
			$user['UserWorkTitle'],
			$user['UserWorkDept'],
			$user['UserWorkOffice'],
			$user['UserWorkAddress'],
			$user['UserWorkCity'],
			$user['UserWorkState'],
			$user['UserWorkZip'],
			$user['UserWorkCountry'],
			$user['UserWorkPhone'],
			$user['UserWorkMobile'],
			$user['UserWorkFax'],
			$user['UserInfo'],
			$user['UserPgpKey'],
			$user['DateModified'])
		);

	}


	function updateshared($user)
	{
		global $atmail;

	  	#$user['id'] = $this->quote($user['id']);

	  	// Update a users addressbook data
	 	$this->db->sqldo( "update Abook_shared set
			UserEmail = ?,
			UserEmail2 = ?,
			UserEmail3 = ?,
			UserEmail4 = ?,
			UserEmail5 = ?,
			UserFirstName = ?,
			UserMiddleName = ?,
			UserLastName = ?,
			UserTitle = ?,
			UserGender = ?,
			UserDOB = ?,
			UserHomeAddress = ?,
			UserHomeCity = ?,
			UserHomeState = ?,
			UserHomeZip = ?,
			UserHomeCountry = ?,
			UserHomePhone = ?,
			UserHomeMobile = ?,
			UserHomeFax = ?,
			UserURL = ?,
			UserWorkCompany = ?,
			UserWorkTitle = ?,
			UserWorkDept = ?,
			UserWorkOffice = ?,
			UserWorkAddress = ?,
			UserWorkCity = ?,
			UserWorkState = ?,
			UserWorkZip = ?,
			UserWorkCountry = ?,
			UserWorkPhone = ?,
			UserWorkMobile = ?,
			UserWorkFax = ?,
			UserInfo = ?,
			UserPgpKey = ?
			where id=?",

			array(
				$user['UserEmail'],
				$user['UserEmail2'],
				$user['UserEmail3'],
				$user['UserEmail4'],
				$user['UserEmail5'],
				$user['UserFirstName'],
				$user['UserMiddleName'],
				$user['UserLastName'],
				$user['UserTitle'],
				$user['UserGender'],
				$user['UserDOB'],
				$user['UserHomeAddress'],
				$user['UserHomeCity'],
				$user['UserHomeState'],
				$user['UserHomeZip'],
				$user['UserHomeCountry'],
				$user['UserHomePhone'],
				$user['UserHomeMobile'],
				$user['UserHomeFax'],
				$user['UserURL'],
				$user['UserWorkCompany'],
				$user['UserWorkTitle'],
				$user['UserWorkDept'],
				$user['UserWorkOffice'],
				$user['UserWorkAddress'],
				$user['UserWorkCity'],
				$user['UserWorkState'],
				$user['UserWorkZip'],
				$user['UserWorkCountry'],
				$user['UserWorkPhone'],
				$user['UserWorkMobile'],
				$user['UserWorkFax'],
				$user['UserInfo'],
				$user['UserPgpKey'],
				$user['id']
				)
		);

		// Next, reload which users have permissions over the table
		// Delete any previous AbookPermission references
		$this->db->sqldo("delete from AbookPermissions where AbookID=?", $user['id']);

		// Add the new results
		$this->addsharedpermissions(array('WriteSelectedGroups' => $user['WriteSelectedGroups'], 'ReadSelectedGroups' => $user['ReadSelectedGroups'], 'WriteSelectedUsers' => $user['WriteSelectedUsers'], 'ReadSelectedUsers' => $user['ReadSelectedUsers'], 'abookid' => $user['id']));

		// Update the shared lookup table, so Outlook clients sync back the latest Webmail changes
		$datemodified = $atmail->outlook_date();
		$this->db->sqldo("update SharedLookup set DateModified=? where LookupID=? and Type='abook'", array($datemodified, $user['id']) );

	}


	/**
	 * Update or create a new entry for a user in the addressbook with a PGP key
	 *
	 * @param array $user Associative array. Valid keys:
	 *                    UserEmail  => users email address
	 *                    UserPgpKey => users PGP key
	 */
	function updatepgp($user)
	{
		global $atmail;

		// make raw email address
		$user['UserEmail'] = preg_replace('/.*?([a-zA-Z0-9!#$%&\'*+\-\/=?^_{|}~.]+@[a-zA-Z0-9.\-]+).*/', '$1', $user['UserEmail']);

		$account = $this->db->sqlgetfield("select UserEmail from $this->Abook where UserEmail=? and Account=?", array($user['UserEmail'], $this->Account));

		// The user account does not exist in the database. Create the account first.
		if (!$account)
			$this->add( array('UserEmail' => $user['UserEmail'], 'UserPgpKey' => $user['UserPgpKey']) );
		else
		{
	    	// Update a users addressbook data
	    	$this->db->sqldo( "update $this->Abook set UserPgpKey = ? where Account = ? and UserEmail=?", array($user['UserPgpKey'], $this->Account, $user['UserEmail']) );
		}
	}


	//returns the public key for one user in the addressbook SQL
	function getPublicKey($recipient)
	{
		global $atmail;

		// try exact match first
	    $key = $this->db->sqlgetfield("select UserPgpKey from $this->Abook where UserEmail=?", $recipient);
	    return $key;
	}

	function clean($var, $num)
	{
	    $newvar = substr( $var, 0, $num );
	    $newvar = rtrim($newvar);
	    if ( count($var) > $num )
	    	$newvar .= ".." ;

	    return $newvar;
	}

	function importparse($num)
	{
		global $pref, $atmail;

		$arr = array();

		if ($fh = @fopen($atmail->tmpdir . $this->Import, 'r'))
		{
			$i = 0;

			$del = $this->_get_csv_delimiter($fh);
			
			while($fields = fgetcsv($fh, 10000, $del))
			{
				if ($num == $i)
					return $arr;

				array_push($arr, $fields);
				$i++;
			}
		}
		else
			print "Cannot open : $atmail->tmpdir$this->Import";

		return $arr;
	}

	function importfile()
	{
		global $pref, $atmail;

		// remove script time limit
		set_time_limit(0);

		// Read the temporary abook file
		if ($fh = fopen($atmail->tmpdir . $this->Import, 'r' ))
		{
		    $count = 0;
		    $line = 0;
		    $is_email = false;

		    $del = $this->_get_csv_delimiter($fh);
		    
		    while (false !== $fields = fgetcsv($fh, 10000, $del))
		    {
		        // Skip if the field contains the header information
                if ($this->ColumnType && $line == 0) {
                	$line++;
                	continue;
                }
                
                $db = array();
                
		        for ($i = 0 ; $i < count($fields); $i++)
		        {
		        	$j = $i + 1;
		            $name = $_REQUEST["ImportField_$j"];
		            
		            if (empty($name)) {
		            	continue;
		            }
		            
		            // Take away any characters which are invalid for the addressbook
		            $fields[$i] = str_replace(array('"', "'"), '', $fields[$i]);

		            $db[$name] = $fields[$i];

		            if (strpos($fields[$i], '@'))
		                $is_email = true;
		        }

		        // Skip if the email address is not defined
		        if ($is_email)
		        {
		            // Finally, add the entry into the database
		            $this->add($db);
		            $count++;
		            $is_email = false;
		        }
		        $line++;
		    }

		    unlink($atmail->tmpdir . $this->Import);
		}
		else
			die ("Cannot open $this->Import");

		return $count;

	}

	function importupload()
	{
		global $pref, $atmail;

		//check file size is not too large
	    if ( $_FILES['fileupload']['size'] > ( $pref['max_msg_size'] * 1048576 ) ) {
	        $this->jsalert = 'csv_import_file_oversize';
	        return false;
	    }

	    // Check file extension
	    if (!preg_match('/\.(csv|txt)$/', $_FILES['fileupload']['name'])) {
	        $this->jsalert = 'csv_import_bad_filetype';
	        return false;
	    }

	    // Lets do a further (lame) check to test that this IS a csv file
	    // Just read in first few lines and check format - we require
	    // at least 2 entries (e.g email_address, first_name) per line
	    $fh = fopen($_FILES['fileupload']['tmp_name'], 'r');
	    
	    $del = $this->_get_csv_delimiter($fh);

	    $row = 0;
        while ($row < 5 && ($data = fgetcsv($fh, 10000, $del)) !== FALSE) {

        	// ignore bank lines
        	if (is_null($data[0]) || count($data) == 1 && empty($data[0])) {
        		continue;
        	}
        	
            if (count($data) < 2) {
                $this->jsalert = 'csv_import_bad_filetype';
                return false;
            }
            $row++;
        }
        fclose($fh);

        // File appears empty
        if ($row == 0 || $row == 1 && isset($_REQUEST['ColumnType'])) {
        	$this->jsalert = 'csv_import_file_empty';
            return false;
        }
        
    	$filename = $_FILES['fileupload']['name'];
		$pathname = AtmailGlobal::escape_pathname($atmail->tmpdir . "{$atmail->Account}-$filename");

		if ( file_exists($pathname) )
		{
			$pathname = AtmailGlobal::escape_pathname($atmail->tmpdir . "$atmail->Account".getmypid().$filename);
		}

		if (move_uploaded_file($_FILES['fileupload']['tmp_name'], $pathname))
		{
			$this->Import = str_replace($atmail->tmpdir, '', $pathname);
		}
		else
		{
			$this->jsalert = 'csv_import_failed';
			return false;
		}


		return true;
	}


	function importsuggest($name, $select)
	{
		global $atmail;
		
		require_once('Language.php');
		
		if ( preg_match('/(First)?\s*Name/i', $name ) || $name == Language::translateAbookField("UserFirstName", $atmail->Language)) 
			$match = "UserFirstName" ;
		elseif ( preg_match('/Last\s*Name/i', $name) || $name == Language::translateAbookField('UserLastName', $atmail->Language)) 
			$match = "UserLastName" ;
		elseif ( preg_match('/Middle\s*Name/i', $name) || $name == Language::translateAbookField('UserMiddleName', $atmail->Language)) 
			$match = "UserMiddleName" ;
		elseif ( preg_match('/(E-mail Address|Email)\s*(\d)?/i', $name, $m) ) 
			$match = "UserEmail{$m[2]}" ;
		elseif ( preg_match('/Job Title/i', $name) || $name == Language::translateAbookField('UserTitle', $atmail->Language)) 
			$match = "UserTitle" ;
		elseif ( preg_match('/Home\s*Address/i', $name) || $name == Language::translateAbookField('UserHomeAddress', $atmail->Language)) 
			$match = "UserHomeAddress" ;
		elseif ( preg_match('/Home\s*City/i', $name) || $name == Language::translateAbookField('UserHomeCity', $atmail->Language)) 
			$match = "UserHomeCity" ;
		elseif ( preg_match('/Home\*State/i', $name) || $name == Language::translateAbookField('UserHomeState', $atmail->Language)) 
			$match = "UserHomeState";
		elseif ( preg_match('/Home Postal Code/i', $name) || $name == Language::translateAbookField('UserHomeZip', $atmail->Language)) 
			$match = "UserHomeZip" ;
		elseif ( preg_match('/Home\s*Country(\/Region)?/i', $name) || $name == Language::translateAbookField('UserHomeCountry', $atmail->Language)) 
			$match = "UserHomeCountry" ;
		elseif ( preg_match('/Home\*Phone/i', $name) || $name == Language::translateAbookField('UserHomePhone', $atmail->Language)) 
			$match = "UserHomePhone" ;
		elseif ( preg_match('/(?<!Work )Mobile( Phone)?/i', $name) || $name == Language::translateAbookField('UserHomeMobile', $atmail->Language)) 
			$match = "UserHomeMobile" ;
		elseif ( preg_match('/Home\*Fax/i', $name) || $name == Language::translateAbookField('UserHomeFax', $atmail->Language)) 
			$match = "UserHomeFax" ;
		elseif ( preg_match('/Personal Web Page|URL/i', $name) || $name == Language::translateAbookField('UserUrl', $atmail->Language)) 
			$match = "UserURL" ;
		elseif ( preg_match('/Company/i', $name) || $name == Language::translateAbookField('UserWorkCompany', $atmail->Language)) 
			$match = "UserWorkCompany" ;
		elseif ( preg_match('/Gender|Sex/i', $name) || $name == Language::translateAbookField('UserGender', $atmail->Language)) 
			$match = "UserGender" ;
		elseif ( preg_match('/Department|Work Dept/i', $name) || $name == Language::translateAbookField('UserWorkDept', $atmail->Language)) 
			$match = "UserWorkDept" ;
		elseif ( preg_match('/Office Location/i', $name) || $name == Language::translateAbookField('UserWorkOffice', $atmail->Language)) 
			$match = "UserWorkOffice" ;
		elseif ( preg_match('/(Business|Work) Street/i', $name) || $name == Language::translateAbookField('UserWorkAddress', $atmail->Language)) 
			$match = "UserWorkAddress" ;
		elseif ( preg_match('/(Business|Work)\s*City/i', $name) || $name == Language::translateAbookField('UserWorkCity', $atmail->Language)) 
			$match = "UserWorkCity" ;
		elseif ( preg_match('/(Business|Work)\s*State/i', $name) || $name == Language::translateAbookField('UserWorkState', $atmail->Language)) 
			$match = "UserWorkState" ;
		elseif ( preg_match('/(Business|Work) Postal Code/i', $name) || $name == Language::translateAbookField('UserWorkZip', $atmail->Language) )
			$match = "UserWorkZip" ;
		elseif ( preg_match('/(Business|Work)\s*Country\/Region/i', $name) || $name == Language::translateAbookField('UserWorkCountry', $atmail->Language)) 
			$match = "UserWorkCountry" ;
		elseif ( preg_match('/(Business|Work)\s*Phone/i', $name) || $name == Language::translateAbookField('UserWorkPhone', $atmail->Language))
			$match = "UserWorkPhone" ;
		elseif ( $name == 'Work Mobile' || $name == Language::translateAbookField('UserWorkMobile', $atmail->Language)) 
			$match = "UserWorkMobile" ;
		elseif ( preg_match('/(Business|Work)\s*Fax/i', $name) || $name == Language::translateAbookField('UserWorkFax', $atmail->Language)) 
			$match = "UserWorkFax" ;
		elseif ($name == Language::translateAbookField('UserInfo', $atmail->Language)) 
			$match = "UserInfo" ;
		elseif ($name == Language::translateAbookField('UserWorkTitle', $atmail->Language)) 
			$match = "UserWorkTitle" ;
		elseif ($name == Language::translateAbookField('UserPgpKey', $atmail->Language)) 
			$match = "UserPgpKey" ;
		elseif ($name == "Date of Birth" || $name == "DOB" || $name == Language::translateAbookField('UserDOB', $atmail->Language)) 
			$match = "UserDOB" ;

		if (isset($match)) {
			$select = str_replace("$match\"", "$match\" selected", $select);
		}
		
		return $select;
	}

	
	/**
	 * Try to detect the csv delimiter
	 * 
	 * This is not fool-proof, just a guess really
	 * The comma get preference by requiring
	 * less hits for a match (also defaults to comma)
	 * upon failure. Perhaps some funky regex that
	 * looks for comma/semi-colon outside of any
	 * quotes would be better??
	 * 
	 * @param $fh resource File handle to csv file
	 * @return String
	 */
	function _get_csv_delimiter($fh)
	{
	    
	    $line = fgetcsv($fh, 10000, ',');
	    if ($line !== false && count($line) > 2) {
	    	$del = ',';
	    }
	    rewind($fh);
	    
	    $line = fgetcsv($fh, 10000, ';');
	    
	    // If $del is already set as ',' then require more hits
	    // for ';' to replace it
	    if ($line !== false && isset($del) && count($line) > 5) {
	    	$del = ';';
	    } elseif ($line !== false && !isset($del) && count($line) > 3) {
	    	$del = ';';
	    }
	    rewind($fh);
	    
	    if (!isset($del)) {
	    	$del = ',';
	    }
	    
	    return $del;
	}


	/**
	 * Check user permissions
	 *
	 * @param int $id
	 * @param string [$user]
	 * @return boolean
	 */
	function check_permissions($id, $user = '')
	{
		global $atmail;

		$id = $this->db->quote($id);
		$permission = false;

		if (strlen($user) > 0 )
		{
			$permission = $this->db->getvalue("SELECT AbookPermissions.Permissions
										   FROM Abook_shared, AbookPermissions
										   WHERE Abook_shared.id = $id AND Abook_shared.UserEmail = $user AND AbookPermissions.AbookID = $id
										   AND (AbookPermissions.Account = $this->Account_Escape OR AbookPermissions.Account = $this->Ugroup
										   		OR AbookPermissions.Account = 'All Users' OR AbookPermissions.Account = $this->pop3host)
										   AND AbookPermissions.Permissions = '1'");

		}
		else
		{

			// Check our account has permissions to delete the addressbook entry
			$permission = $this->db->getvalue("SELECT AbookPermissions.Permissions
										   FROM Abook_shared, AbookPermissions
										   WHERE Abook_shared.id = $id AND AbookPermissions.AbookID = $id
										   AND (AbookPermissions.Account = $this->Account_Escape OR AbookPermissions.Account = $this->Ugroup
										   		OR AbookPermissions.Account='All Users' OR AbookPermissions.Account=$this->pop3host)
										   AND AbookPermissions.Permissions ='1'");
		}

		return $permission;
	}


	// Check to see if a group already exists
	function checkgroup($group)
	{
		global $atmail;
        $abookGroup = Filter::cleanSqlFieldNames($this->db->AbookGroup);
		$group = $this->db->sqlgetfield("select GroupName from $abookGroup where GroupName=$this->Ugroup
		and Account=$this->Account_Escape");

		return $group;
	}


	// LDAP search functions
	function _search_ldap($db)
	{
		// Check for PHP LDAP extension
		if (!defined('LDAP_OPT_TIMELIMIT'))
			return array();

		global $atmail, $pref;

		if (empty($db['Account']) && empty($db['FirstName']) && empty($db['LastName']))
			return array();

		if (!@$ldap = ldap_connect($pref['ldap_server'], 389))
			return array();

		if (!@ldap_bind($ldap, $pref['bind_dn'], $pref['ldap_password']))
			return array();

		$query = "(& ";

		if (!empty($db['Account']))
	        $query .= "(mail={$db['Account']}*) ";

		if (!empty($db['FirstName']) && !empty($db['LastName']))
	        $query .= "(cn={$db['FirstName']} {$db['LastName']}*) ";

		if (!empty($db['FirstName']))
			$query .= "(givenName={$db['FirstName']}*) ";

		if (!empty($db['LastName']))
			$query .= "(sn={$db['LastName']}*) ";

		$query .= ")";

	    // Now we make our query to the LDAP server according to the users
	    // input via the form
		$result = @ldap_search($ldap, $pref['base_dn'], $query );

		if ($result === false) {
		    $atmail->ldap_error("$query: ". ldap_error($ldap));
		} else {
		    foreach (@ldap_get_entries($ldap, $result) as $entry)
		    {
		        if (!is_array($entry))
		        continue;

		        $email = $entry['mail'][0];

		        $h[$email]['Account'] = $entry['mail'][0];
		        $h[$email]['UserLastName'] = $entry['sn'][0];
		        $h[$email]['UserFirstName'] = $entry['givenname'][0];
		    }

		    return $h;
		}
	}

}

?>
