<?php
// +----------------------------------------------------------------+
// | AdminBase.php  												|
// +----------------------------------------------------------------+
// | Source-code (C) CalaCode.com 1998-2005 . All rights reserved	|
// | See http://calacode.com/license.ehtml for license agreement	|
// +----------------------------------------------------------------+
// | Date: Febuary 2005												|
// +----------------------------------------------------------------+

require_once('header.php');

require_once('GetMail.php');
require_once('Config.php');
require_once('Global.php');
require_once('Log.php');
require_once('Alias.php');
require_once('Auth.php');

define('ATMAIL_SUPER_ADMIN', 1);
define('ATMAIL_SUB_ADMIN',   2);

class AdminBase
{
	/**
	 * The operating mode: ATMAIL_SUPER_ADMIN or ATMAIL_SUB_ADMIN
	 *
	 * @var int
	 */
	var $mode;

	// associative array used to store variables (name => value) used in the html templates
	var $vars = array();

	var $Username;
	var $Password;
	var $domains = array();
	var $NumQuota;
	var $adminquery;
	var $unixrestart;


	function AdminBase($mode)
	{
	    global $atmail;
	    $atmail->adminmode = $mode;
		$this->mode = $mode;

		$this->log = new Log;

		$this->unixrestart = '';//"Restart the Apache Webserver to insure the mod-perl cache is reloaded with the new settings" if($^O !~ /Win/);
	}


	/**
	 * Upload the migrate file to the server
	 *
	 * @param string $filename the filename of the migrate file
	 * @access private
	 * @return mixed string error message if filename not specified.
	 *               void on success.
	 *               Exits script if $filename cannot be saved
	 */
	function _attach($filename)
	{
		global $pref, $atmail;

		if (!$atmail->isset_chk($filename) && strlen($filename) == 0)
			return "File not specified!";

		if (!move_uploaded_file($filename, "{$pref['user_dir']}/tmp/migrate.txt"))
			die("Cannot save migrate on server");
	}


	function _get_date($timestamp = null)
	{
		if (!$timestamp)
			$timestamp = time();

		$ltime = localtime($timestamp, true);

		// $ltime['tm_year'] contains years since 1900
		// so we need to add 1900 to get current year
		$year = $ltime['tm_year'] + 1900;

		$month = $ltime['tm_mon'];

		if ( strlen($month) == 1 )
		    $month++;
			$month = "0$month";

		$day = $ltime['tm_mday'];
		if ( strlen($day) == 1 )
			$day = "0$day";

		$time = $ltime['tm_hour'].$ltime['tm_min'].$ltime['tm_sec'];

		return array($year, $month, $day, $time);
	}


	function _deleteuser($user)
	{
		global $sql, $atmail;

		// Load the table names
		$sql->table_names($user);

		// Print some output, so the browser will not expire
		print " ";

		// Loop through the common tables of user accounts
		foreach (array('EmailUIDL', 'SpamDB', 'MailSort', 'Abook', 'UserSettings', 'UserPgp', 'AbookGroup') as $v)
			$sql->sqldo("delete from {$sql->$v} where Account=?", $user);

		// Delete all the messages in the database from the specified user
		$ids = $sql->sqlarray("select id from $sql->EmailDatabase where Account=?", $user);

		foreach ($ids as $id)
		{
			$sql->sqldo("delete from $sql->EmailMessage where id='$id'");
			$sql->sqldo("delete from $sql->EmailDatabase where id='$id'");
		}

		// Delete the maildir if required
		$maildir = $sql->sqlgetfield("select MailDir from Users where Account=?", $user);
		if ($atmail->isset_chk($maildir))
			$this->_delete_user_maildir($maildir);

		// Delete the remaining tables
		$sql->sqldo("delete from Users where Account=?", $user);
		$sql->sqldo("delete from UserSession where Account=?", $user);
		$sql->sqldo("delete from $sql->Calendar where UserTo=?", $user);
		$sql->sqldo("delete from Folders where Account=?", $user);
		$sql->sqldo("delete from SpamSettings where username=?", $user);
		return;
	}


	function _delete_user_maildir($user)
	{
		global $domains, $atmail;

		$user = preg_replace('/\.\.|\0/', '', $user);

		// Return if there is no username or specified user is not a directory
		if ( !$user || !is_dir($user) || strpos($user, '@') === false )
			return;

		// We are running Windows ...
		if ( strpos(PHP_OS, 'WINNT') !== false )
		{
			$path = "$user/";
			$path = str_replace('/', '\\', $path);
			`rd /s /q $path`;
		}
		else
		{
			// Unix style delete
			$atmail->rmtree($user);
		}
	}


	function _searchusers($query, $func)
	{
		global $atmail, $sql;

		$this->vars['func'] = $func;
		$this->vars['userlist'] = '';

		// define extra sql statement needed if running under subadmin
		if ($this->mode == ATMAIL_SUB_ADMIN)
				$extra = "and $this->adminquery ";
		else
			$extra = '';

		if ($atmail->isset_chk($_REQUEST['id']) && is_array($_REQUEST['id']))
		{
			foreach($_REQUEST['id'] as $id)
			{
				if (!$id)
					continue;

				$this->vars['userdelete']++;

				$this->_deleteuser( $id );
			}
		}

		if ( !empty($query) )
		{
			if (preg_match('/^insert/i', $query) || strpos($query, 'delete') !== false || strpos($query, 'update') !== false)
				die("Security alert - Invalid query");

			//$query = "select id from Users where id is not null $extra" . $query;
			$query .= $extra;

			$this->vars['status']  = "Search Results";
			$this->vars['comment'] = $atmail->parse("msgs/comment-searchusers.html");
		}
		else
		{
			$this->vars['status']  = "Listing all users";
			$this->vars['comment'] = $atmail->parse("msgs/comment-allusers.html");

			if ($this->mode == ATMAIL_SUPER_ADMIN)
				$query = "select Users.id from Users,UserSession where Users.Account = UserSession.Account";
			else
				$query = "select Users.id from Users,UserSession where Users.Account = UserSession.Account and $this->adminquery";

		}

		$this->vars['Order'] = ($atmail->isset_chk($_REQUEST['Order'])) ? $_REQUEST['Order'] : 'id';
		$this->vars['Desc']  = ($atmail->isset_chk($_REQUEST['Desc']))  ? $_REQUEST['Desc']  : 'asc';

		if (strpos($query, 'order by') === false)	{

			if ($this->vars['Order'] == "LastLogin")	{
                $this->vars['Order'] = "UserSession.LastLogin";
                $query = str_replace('Users', 'Users, UserSession', $query);
                $query .= ' AND Users.Account = UserSession.Account ';
			} elseif (!preg_match('/Users\./', $this->vars['Order']) && $this->vars['Order'] != "LastLogin"  && $this->vars['Order'] != "UserSession.LastLogin") {
                $this->vars['Order'] = "Users.{$this->vars['Order']}";
			}

			$order = Filter::cleanSqlFieldNames($this->vars['Order']);
			$desc = Filter::stringMatch($this->vars['Desc'], array('desc', 'asc'));

			$query .= " order by $order $desc";
		}

		$id = $sql->sqlarray_fast($query);

		// The total number of log entries
		$this->vars['total'] = count($id);
		$this->vars['status'] .= " - {$this->vars['total']} records";

		$this->vars['num'] = ($atmail->isset_chk($_REQUEST['num'])) ? $_REQUEST['num'] : 25;

		$this->vars['numselect'] = <<<_EOF
		<select class='option' name="num" onchange='uncheckall();document.atmail.submit();'>
		<option value='25'>25</option>
		<option value='50'>50</option>
		<option value='100'>100</option>
		<option value='200'>200</option>
		<option value='500'>500</option>
		<option value='1000'>1000</option>
		</select>
_EOF;

		$this->vars['numselect'] = str_replace("value='{$this->vars['num']}'", "value='{$this->vars['num']}' selected", $this->vars['numselect']);
		$this->vars['msg_pos'] = ($atmail->isset_chk($_REQUEST['start']) && is_numeric($_REQUEST['start'])) ? $_REQUEST['start'] : $this->vars['total'] - $this->vars['num'];

		// Create the next / prev buttons if nessasary
		if ( $this->vars['msg_pos'] + $this->vars['num'] < $this->vars['total'] )
		{
			$next_pos = $this->vars['msg_pos'] + $this->vars['num'];

			$this->vars['next'] = <<<_EOF
		<a href="javascript:searchusers('{$this->vars['type']}','$next_pos')">
		<img src="imgs/next.gif" border=0 alt="Next messages">
		</a>
_EOF;
		}

		if ( $this->vars['total'] - $this->vars['num'] >= 0 && $this->vars['msg_pos'] != 1 )
		{
			$prev_pos = $this->vars['msg_pos'] - $this->vars['num'];
			if ( $prev_pos <= 0 )
				$prev_pos = 1;

			$this->vars['prev'] = <<<_EOF
	<a href="javascript:searchusers('{$this->vars['type']}','$prev_pos')">
	<img src="imgs/previous.gif" border=0 alt="Previous messages">
	</a>
_EOF;
		}

		// Find which messages to display, depending if the user clicked 'next/prev' buttons
		if ( $this->vars['msg_pos'] + $this->vars['num'] > $this->vars['total'] )
			$this->vars['msg_pos'] = $this->vars['total'] - $this->vars['num'];

		if ( $this->vars['msg_pos'] <= 1 )
			$this->vars['msg_pos'] = 0;

		if ( $this->vars['msg_pos'] + $this->vars['num'] <= $this->vars['total'] )
			$this->vars['total'] = $this->vars['msg_pos'] + $this->vars['num'];

		// Create a temp hash containing the users details

		// List messages from the newest to oldest
		for ($i = $this->vars['total'] ; $i >= $this->vars['msg_pos'] ; $i-- )
		{
			$myid = $id[$i];
			if ( !$myid )
				continue;

			$this->vars['class'] = ( $this->vars['class'] == "odd" ) ? "even" : "odd";

			$db = $sql->sqlhash("select Account,DateCreate from Users where id=?", $myid);

			$this->vars['time'] = $sql->getvalue("select LastLogin from UserSession where Account=?", $db['Account']);

			//fix this
			$this->vars['time'] = strftime( "%a %D %R", $this->vars['time']);

			$this->vars['Account'] = $db['Account'];

			// Delete any whitespace , to avoid breaking the javascript.
			$this->vars['Account'] = preg_replace('/\s/', '', $this->vars['Account']);

			// Parse the date into a more reabable format
			$db['DateCreate'] = preg_replace('/\d\d(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/', '$2/$3/$1 $4:$5:$6', $db['DateCreate']);

			$this->vars['userlist'] .= $atmail->parse("html/user_entry.html", array_merge($db, $this->vars));
		}


		print $atmail->parse( "html/users.html", $this->vars );
	}


	function _makesql_query($var=array())
	{
		global $atmail, $sql;

		$extra = "";

		// Loop through each field
		foreach (array('Account', 'Ugroup','FirstName', 'LastName', 'Address', 'City', 'State',
		  'PostCode', 'Country', 'OtherEmail', 'Gender', 'OtherEmail', 'Industry',
		  'Occupation', 'BirthDay', 'BirthMonth', 'BirthYear', 'UserQuota') as $name)
		{
			// Load the field search type
			$search = $name . "_o";

			// Load the value of the search field
			$value = $atmail->param($name);

			$var[$search] = $atmail->param($search);

			$var['hidden'] .= "<input type='hidden' name='$name' value='$value'>\n";
			$var['hidden'] .= "<input type='hidden' name='$search' value='$var[$search]'>\n";

			if ( $atmail->param('pop3host') && !$atmail->param('Account') && $name == "Account" )
				$value = '%';

			// Add in % for like querys
			if ( $var[$search] == "like" || $var[$search] == "not like" )
			{
				if ( !$value ) continue;
				$value = "%" . $value . "%";
			}

			if ( $atmail->param('pop3host') && $name == "Account" )
			{
				$pop3host = $atmail->param("pop3host");
				$value = "$value@$pop3host";
				$var['hidden'] .= "<input type='hidden' name='pop3host' value='$pop3host'>";
                $var[$search] = ($_REQUEST['where'] == 'notlike') ? 'not like' : 'like';
			}

			// Skip if we have no value and no search query
			if (!$var[$search] || !$value) continue;

			// SQL quote the query
			$var[$name] = $sql->quote($value);

			// Create our extended SQL query
			if ( $var[$name] )
				$extra .= "$name $var[$search] $var[$name] and ";
		}

		// Take away the last and statement

		if (strlen($extra) > 0) {
			$extra = 'WHERE '.$extra;
		}
		$extra = preg_replace('/and $/', '', $extra);

		$var['query'] = "select id from Users $extra";

		return $var;
	}


	function _group_options()
	{
		global $groups;

		$txt = '';
		foreach (array_keys($groups) as $group )
			if($group == 'Default')
			$txt .= "<option value=\"$group\" selected>$group</option>";
			else
			$txt .= "<option value=\"$group\">$group</option>";

		return $txt;
	}


	function _calc_week($time=null)
	{
		$days = 86400 * 7;

		$time = $time - $days;

		$newtime = strftime( "%D %T", $time);
		$newtime = preg_replace('/(\d\d)\/(\d\d)\/(\d\d) (\d\d):(\d\d):(\d\d)/', '$3$1$2$4$5$6', $newtime);

		return $newtime;
	}

	function _smtp_type()
	{
		global $pref;

		$txt = <<<_EOF
			<select class="option" name="smtp_type">
			  <option value="POP3">WebMail Access Only</option>
			  <option value="Exim">Exim ( Default Server Mode MTA )</option>
			</select>
_EOF;

		$txt = str_replace("option value=\"{$pref['smtp_type']}\"", "option value=\"{$pref['smtp_type']}\" selected", $txt);

		// For our Windows friends...
		if( strpos(PHP_OS, 'WINNT') !== false && $pref['smtp_type'] == "Exim")
			$txt = "<input type='hidden' name='smtp_type' value='Exim'>";

		return $txt;
	}


	function _install_type()
	{
		global $pref;

		$txt = <<<_EOF
		<select class="option" name="install_type">
		<option value="standalone">Email Client</option>
		<option value="server">Email Server</option>
		</select>
_EOF;

		$txt = str_replace("option value=\"{$pref['install_type']}\"", "option value=\"{$pref['install_type']}\" selected", $txt);

		return $txt;
	}

	function _mail_type()
	{
		global $pref;

		$txt = <<<_EOF
		<select class="option" name="mail_type">
		<option value="pop3imap">POP3 and IMAP</option>
		<option value="pop3">POP3 only</option>
		<option value="imap">IMAP only</option>
		<option value="none">No External Servers</option>
		</select>
_EOF;

		$txt = str_replace("option value=\"{$pref['mail_type']}\"", "option value=\"{$pref['mail_type']}\" selected", $txt);

		return $txt;
	}

	function _sendmode()
	{
		global $pref;

		$txt = <<<_EOF
		<select class="option" name="sendmode">
		<option value="pipe">Sendmail Binary</option>
		<option value="smtp">SMTP Server</option>
		</select>
_EOF;

		$txt = str_replace("option value=\"{$pref['sendmode']}\"", "option value=\"{$pref['sendmode']}\" selected", $txt);

		return $txt;
	}

	// Check for any errors with the Configuraiton
	function _check_errors()
	{
		global $pref;

		// init error variable
		$error = '';

		// Test our @Mail installation is functioning

		// Get the users running @Mail, only under Unix
		//fix this
		if ( strpos(PHP_OS, 'WINNT') === false )
			$var['user'] = getmyuid();

		// See if we can open to our @Mail direcfory
		foreach (array($pref['user_dir'], "{$pref['user_dir']}/tmp", "{$pref['user_dir']}/users/") as $dir )
		{
			if (!is_writeable($dir))
				$error .= "<font class=error><li> Cannot write to the User Directory of @Mail $dir . Please check the directory exists and change the ownership of the directory to your webserver user {$var['user']} (chown -R {$var['user']} $dir)</li></font>";
		}

		// Next, check our sendmail binary or SMTP server is valid
		if ( $pref['sendmode'] == "pipe" )
		{
			$sendmail = $pref['sendmail'];
			$sendmail = preg_replace('/ .*/', '', $sendmail);

			// If we cannot see the sendmail binary, report the error
			if (!file_exists($sendmail))
			{
				$error .= "<font class=error><li> The sendmail binary {$pref['sendmail']} could not be located on your system. Check you have provided the correct pathname. Otherwise, use an external SMTP server to deliver emails</li></font>";
			}

			// Check the SMTP server is valid, and can be found
		}
		else
		{
			require_once('Net/SMTP.php');

			$smtp = new Net_SMTP($pref['smtphost']);

			if (PEAR::isError($smtp->connect(60)))
			{
				$error .= "<font class=error><li> The SMTP server would not connect. Please check the domain-name is correct, you have access to the SMTP server and your connection to the network is active</li></font>";
			}

			// Check SMTP authentication details are correct
			if ($pref['smtpauth_username'] && $pref['smtpauth_password'])
				if($smtp->auth($pref['smtpauth_username'], $pref['smtpauth_password']) !== true)
					$error .= "<font class=error><li> The SMTP authentication details provided are incorrect. Please verify you have the correct username and password. Check your SMTP configuration for further details, or use an SMTP server which you have IP relay permissions.</li></font>";

			$smtp->disconnect();
		}

		// Check the connection to the database server is active
		if ( $pref['sql_type'] )
		{
			global $sql;

			if (!is_object($sql))
			{
				$error .= "<font class=error><li> @Mail could not connect to the database server. Check the database name is correct, the database exists, and the username/password are correct to connect. Verify you are using the correct database driver for your system</font>";
			}
		}

		// Check that the @Mail directory is owned by the webserver
		$dirs = $pref['install_dir'];

		$dirs = str_replace('/webadmin', '', $dirs);

		foreach ( array("$dirs/libs/Atmail/", $dirs) as $dir )
		{
			if (!is_writeable($dir))
				$error .= "<font class=error><li> The $dir directory does not have write permissions. Change the ownership of the directory to your webserver user {$var['user']} . (chown -R {$var['user']} $dir)</li></font>";
		}

		if ($pref['gpg_path'] && !file_exists($pref['gpg_path']))
		{
			$error .= "<font class=error><li> The {$pref['gpg_path']} cannot be found. Please specify the correct pathname for the GNU PGP binary. Download a copy from <a href='http://www.gnupg.org/' target='_blank'>http://www.gnupg.org/</a></li>";
		}

		if ($pref['gpg_path'] && file_exists($pref['gpg_path']))
		{
			$text = `{$pref['gpg_path']} --help`;

			preg_match('/gpg \(GnuPG\)\s([\d\.]+)/', $text, $match);

			$version = $match[1];
			$dotversion = str_replace('.', '', $version);

			if ($dotversion < 121 && $version)
			{
				$error .= "<font class=error><li>The system required version 1.2.1 or greater of GNU PGP. The system is currently running version $version. Please upgrade the GNU PGP installation.</li>";
			}

			if (strpos($text, 'Supported algorithms:') === false)
			{
				$error .= "<font class=error><li> The system does not recognize {$pref['gpg_path']} as a valid GNU PGP binary. Please check the install of {$pref['gpg_path']} and try again.</li>";
			}
		}

		if (strpos($pref['user_dir'], 'tmp') !== false)
		{
			$error .= "<font class=error><li>The user directory {$pref['user_dir']} is invalid. Please specify a directory that does not contain the string 'tmp'.</font></li>";
		}


		if ($pref['install_type'] == "server")
		{
			// Open the user directory
			if ($dir = opendir("/usr/local/atmail/mailserver/bin/" ))
			{
				$eximbin = null;

				while (false !== $fol = readdir($dir))
				{
					if (strpos($fol, 'exim-4.') !== false)
					{
						$eximbin = $fol;
						break;
					}
				}
			}
			else {
				$error .= "Cannot open /usr/local/atmail/mailserver/bin/";
			}

			// Check the exim binary exists ( skip if client mode )
			if (!is_file("/usr/local/atmail/mailserver/bin/$eximbin")) {
			//&& $pref['install_type'] == "server") {
				$error .= "Cannot open /usr/local/atmail/mailserver/bin/$eximbin";
			} else {
			   $owner = fileowner("/usr/local/atmail/mailserver/bin/$eximbin");
			   $fstat = stat("/usr/local/atmail/mailserver/bin/$eximbin");
			}

		}


		if (($pref['install_type'] != "standalone") && ($owner != "0"))
		{
			//try to get name of owner from uid
			if (function_exists('posix_getpwuid'))
			{
				$owner = posix_getpwuid($owner);
				$owner = $owner['name'];
			}
			$error .= "<font class=error><li>The Mailserver binary has incorrect ownership. Currently set as the user $owner - /usr/local/atmail/mailserver/bin/$eximbin - Please chown the binary as the 'root' user</font></li>";
		}

		if($fstat['mode'] != '35181' && $pref['install_type'] != "standalone" )	{

			$error .= "<font class=error><li>The Mailserver binary has incorrect permissions. Currently not set as setuid for restart permissions - Run via the terminal as root: chown root {$pref['user_dir']}/mailserver/bin/$eximbin; chgrp atmail {$pref['user_dir']}/mailserver/bin/$eximbin ; chmod 4555 {$pref['user_dir']}/mailserver/bin/$eximbin</font></li>";

		}

		// Test PHP safe mode is off
		if(ini_get('safe_mode'))	{
			$error .= "<font class=error><li>PHP Safe mode is turned on. This must be disabled for the correct operation of @Mail. Edit the php.ini and disable Safe-mode support, then restart the Webserver for changes to take effect</font></li>";

		}

		 if (strlen($error) > 0)
			 $error = "<font class=error><ul><li><b>@Mail Configuration Errors</b></li>$error</ul></font>";

		return $error;
	}

	function _mysqlversion()
	{
		global $sql;

		$mysqlversion = $sql->getvalue("select version()");

		return $mysqlversion;
	}

	function _mysqltable()
	{
		global $pref;

		foreach ( array('/usr/local/mysql/data', '/usr/local/var') as $dir )
		{
			if (is_dir($dir))
			{
				$mysqltable = "$dir/{$pref['sql_table']}";
				return $mysqltable;
			}
		}
		return;
	}

	function _tarlocation()
	{
		foreach ( array('/usr/local/bin/tar', '/usr/bin/tar', '/bin/tar', '/usr/local/tar/bin/tar') as $path)
		{
			if (file_exists($path))
			{
				$tarlocation = "$path cfvz";
				return $tarlocation;
			}
		}
		return;
	}

	function _mysqldumplocation()
	{
		foreach (array('/usr/bin/mysqldump', '/usr/local/mysql/bin/mysqldump', '/usr/local/bin/mysqldump', '/usr/mysql/bin/mysqldump') as $path)
		{
			if (file_exists($path))
				return $path;
		}
		return;
	}

	function _logtype($log, $type)
	{
		if ( $type == "Login" || $type == "Error" )
			$log = "$log$type";

		elseif ( $type == "RecvMail" )
			$log = "Msg" . $type;

		elseif ( $type == "SentMail" )
			$log = "Sent" . $type;

		return $log;
	}

	function _dberror($msg)
	{

	print <<<_EOF
	<html>
	<head>
	</head>
	<script language="JavaScript" src="js/testagent.js"></script>
	<body background="../imgs/watermark.gif" bgColor="#ffffff" vlink="#000000" link="#5167C6" alink="#6666FF" topmargin="0" leftmargin="5">
	<table width="100%">
	  <tr>
		<td>
		  <hr>
		  <font class="header">Database Connection Error</font>
		</td>
	  </tr>
	  <tr>
		<td>
		  <font class="comment">
	Could not connect to the database server. Error message: <b>$msg</b><br><br>Please verify the database connection details. Check the username and password
		are correct, and create the database name if required.</a>
	</font>
		<br>
		</td>
	  </tr>
	  <tr>
		<td>
		  <hr>
		</td>
	  </tr>
	</table>
	</body>
	</html>
_EOF;

	}

	function _dbierror()
	{

	print <<<_EOF
	<html>
	<head>
	</head>
	<script language="JavaScript" src="js/testagent.js"></script>
	<body background="../imgs/watermark.gif" bgColor="#ffffff" vlink="#000000" link="#5167C6" alink="#6666FF" topmargin="0" leftmargin="5">
	<table width="100%">
	  <tr>
		<td>
		  <hr>
		  <font class="header">DBI Database module not installed</font>
		</td>
	  </tr>
	  <tr>
		<td>
		  <font class="comment">
	The DBI database module is not installed. Please use the installation utility to switch
	@Mail to use a database backend, and consult the online documentation.</a>
	</font>
		<br>
		</td>
	  </tr>
	  <tr>
		<td>
		  <hr>
		</td>
	  </tr>
	</table>
	</body>
	</html>
_EOF;

	}

	function _escape_html($string)
	{
		return htmlentities($string);
	}

	function _unescape_html($string)
	{
		return html_entity_decode($string);
	}

	function _purgelogs($log, $days)
	{
		global $sql;

		$log = Filter::cleanSqlFieldNames($log);
		if (!is_numeric($days))
            $days = 1;

		// Query the number of records that match
		$date = $sql->makedatetime($days * -1, 'day');
		
		$count = $sql->sqlgetfield("select count(id) from $log where LogDate < $date");

		// Next, purge the logs if the the count is > 0
		if ($count > 0)
			$sql->sqldo("delete from $log where LogDate < $date");

		// Optimize the table after a purge
		$sql->optimize_table($log);

		return $count;
	}

	/**
	 * Display splash screen
	 * @access public
	 */
	function splash()
	{
		global $atmail;
		print $atmail->parse("html/adminheader.html");
		print $atmail->parse( "html/splash.html", $this->vars);
	}

	/**
	 * Display 'Modify User' screen
	 * @access public
	 */
	function modifyuser()
	{
		global $atmail;
		print $atmail->parse("html/adminheader.html");
		print $atmail->parse("html/modifyuser.html", $this->vars);
	}

	/**
	 * Display 'Search Users' screen
	 * @access public
	 */
	function searchusers()
	{
		global $atmail;

		print $atmail->parse("html/adminheader.html");

		$this->vars['Groups']    = $this->_group_options();
		$this->vars['domainbox'] = $atmail->domainbox();
		print $atmail->parse( "html/searchusers.html", $this->vars );
	}

	/**
	 * Search users via the SQL database
	 *
	 * @access public
	 */
	function searchinguser()
	{
		global $atmail, $func;


		if (!$atmail->isset_chk($_REQUEST['query'])) {
		    $var = $this->_makesql_query($this->vars);
		}
		elseif ($this->mode == ATMAIL_SUPER_ADMIN) {
		    $var['query'] = $_REQUEST['query'];
		}

		// If the SQL query is empty
		if (preg_match('/where $/i', $var['query']))
		{
			$this->vars['status'] = "No query specified";

			if ($this->mode == ATMAIL_SUPER_ADMIN) {
				$var['query']  = "select id from Users";
			}
			else
				$var['query']  = "select id from Users where $admin->adminquery";
		}

		// Subadmin search with extra parameters, add our adminquery
		elseif ($this->mode == ATMAIL_SUB_ADMIN && preg_match('/WHERE /i', $var['query']) ) {
		    $var['query'] .= " and $this->adminquery";
        }

		// Search without any custom fields
		elseif ($this->mode == ATMAIL_SUB_ADMIN ) {
		    $var['query'] .= "WHERE 1 and $this->adminquery";
        }


		print $atmail->parse( "html/adminheader.html", $this->vars );
		print $atmail->parse("html/loading.html");
		$this->_searchusers( $var['query'], $func);
	}

	/**
	 * Creates Database structure
	 * @access public
	 */
	function cds()
	{
		global $sql, $atmail, $pref;

		// Open the install_size SQL query
		$fh = fopen("../docs/{$pref['install_size']}.sql", 'r');

		$file = '';

		// Load the SQL statments into a temp var
		while (false !== $line = fgets($fh))
		{
			// Skip comments
			if (preg_match('/^#|^-/', $line))
				continue;

			$file .= $line;
		}

		// Split each SQL query into bits
		$sql_parts = explode(';', $file);

		$tables = $sql->tables();

		foreach ($sql_parts as $query)
		{
			// Get the Create table name
			if ( preg_match('/CREATE TABLE (.*) \(/i', $query, $m) )
				$name = $m[1];

			print "<b>.</b>";

			if ( !$name )
				continue;

			if ( !in_array($name, $tables))
			{
				// Creating the structure
				$sql->sqldo($query);
				$this->vars['db'] .= "$name created<BR>";
			}
			else
				$this->vars['db'] .= "$name already exists <br>";
		}

		$this->vars['title'] = "Create Database Structure";
		print $atmail->parse( "html/adminheader.html", $this->vars );
		print $atmail->parse( "html/cds.html",         $this->vars );
	}


	function users()
	{
		global $atmail, $func;
		print $atmail->parse("html/adminheader.html");
		print $atmail->parse("html/loading.html");

		// Yikes, we have to clean this one up, no SQL query should be allowed
		$query = ($atmail->isset_chk($_REQUEST['sql'])) ? $_REQUEST['sql'] : '';

		$this->_searchusers($query, $func);
	}

	function notyet()
	{
		global $atmail;
		print $atmail->parse( "html/adminheader.html", $this->vars );
		print $atmail->parse("html/notyet.html");
	}


	/**
	 * Build a query that identifies the admin account
	 *
	 * @access public
	 */
	function adminquery()
	{
		global $sql;

		// Find the Admin domains they can control
		$domains = $sql->sqlarray("select Domain from AdminGroup where Username=? and Domain is not null", $this->Username);
		$this->adminquery = "(";

		foreach($domains as $dom)
		{
			$this->domains[] = $dom;
			$domain = $sql->quote("%@$dom");
			$this->adminquery .= "Users.Account like $domain or ";
		}

		// Find the Groups the admin can control
		$groups = $sql->sqlarray("select Ugroup from AdminGroup where Username=? and Ugroup is not null", $this->Username);

		$extra = '';

		foreach ($groups as $grp)
		{
		    if (empty($grp))
		       continue;
			$grp = $sql->quote($grp);
			$extra .= "Users.Ugroup = $grp or ";
		}

		if (!empty($extra)) {
			$this->adminquery = preg_replace('/or $/', ') and (', $this->adminquery);
			$this->adminquery .= $extra;
		}

		// Take away the last 'or' from the query
		$this->adminquery = preg_replace('/or $/', ')', $this->adminquery);
	}

	/**
	 * Find total used quota for all accounts
	 *
	 * @access public
	 */
	function checkquota()
	{
		global $sql;

		$totalquota = 0;

		if (!$this->NumQuota)
			return 0;

		$accounts = $sql->sqlarray("select Account from Users where $this->adminquery");

		// Loop through each user and query the size of their mailbox ( total allocated quota )
		foreach ($accounts as $acc)
			$totalquota += $sql->getvalue("select UserQuota from Users where Account=?", $acc);

		return $totalquota;
	}


	/**
	 * Set $pref['hostname'] to the hostname of the server
	 *
	 * @access private
	 */
	function _checkhostname()
	{
		global $pref;
		$pref['hostname'] = php_uname('n');
	}


	/**
	 * Create dropdown of Global addressbook
	 *
	 * @return string
	 * @access private
	 */
	function _globalabook($selected_value)
	{
		$txt = <<<_EOF
		<select name="GlobalAbook" class="option">
	  <option value="0">Restricted</option>
	  <option value="1">Domain</option>
	  <option value="2">Site</option></select>
_EOF;

		$txt = str_replace("option value=\"$selected_value\"", "option value=\"$selected_value\" selected", $txt);

		return $txt;
	}


	/**
	 * Wrapper for AtmailGlobal::parse. Adds in warning if webadmin insecure
	 */
	 function parse($file, $vars=array())
	 {
	 	global $atmail, $pref;

	 	if (!$this->_htaccessRead())
	 	{
			$warning = '<p><strong>The Webadmin is insecure! Please <a href="admin.php?func=htpasswd">secure webadmin</a> immediately with a password. Validate Apache is setup for .htaccess support to the atmail directory for the password protection to work</strong></p>';

			if (isset($vars['status']))
				$vars['status'] = $warning . $vars['status'];
			else
				$vars['status'] = $warning;
		}

		if (!file_exists($pref['install_dir'].'/tmp/.htaccess'))
		{
			$warning = '<p><strong>The temp files are insecure! Please, create .htaccess file at '.$pref['install_dir'].'/tmp/ contains "deny from all"  and validate Apache is setup for .htaccess support to that directory</strong></p>';

			if (isset($vars['status']))
			{
				$vars['status'] = $warning . $vars['status'];
			}
			else
			{
				$vars['status'] = $warning;
			}

		}

		return $atmail->parse($file, $vars);
	}

	/**
	 * try to determine if Apache has read .htaccess file
	 *
	 * @return bool
	 */
	function _htaccessRead()
	{
        // Skip displaying if Subadmin user
        if($this->mode == 2)
        return 1;

		return (isset($_SERVER['REMOTE_USER']) && isset($_SERVER['AUTH_TYPE']));
	}

	/**
	 * Function to add a new domain into Config.php and the local DB
	 *
	 * @return bool
	 */

	function _newdomain($dom)
	{
		global $domains, $sql;

		if (!$domains[$dom]) {
            $domains[$dom] = 1;
            writeconf();
		}

		// Next, add all the domains into the SQL database that is read by Exim
		if (!$sql->sqlgetfield("select Hostname from Domains where Hostname=?", array($dom) ) ) {
			$sql->sqldo("insert into Domains (Hostname) VALUES (?)", array($dom) );
		}

		return 0;
	}

function _addsubadmin_user($var)        {

        global $sql;

        // Check if the user already exists
        $uname = $sql->getvalue("select Username from AdminUsers where Username=?", $var['Username']);

        // Account exists
        if ($uname)
        	return 1;

        // Check the user has a valid password
        if (!$var['Password'])
        {
        	return 2;
        }

        $var['NumUsers'] = ($var['NumAccounts'])? $var['NumAccounts'] : $var['NumUsers'];
        $var['NumQuota'] = (!is_numeric($var['NumQuota']) || intval($var['NumQuota']) < 0) ? 0 : $var['NumQuota'];
        $var['BrandDomain'] = $varT['BrandDomain'] ? $var['BrandDomain'] : '';

        $data = array($var['Username'], $var['Password'], $var['Company'], $var['Fullname'],
        $var['NumUsers'], $var['UAdd'], $var['UModify'], $var['UPurge'],
        $var['USearch'], $var['UList'], $var['UMigrate'], $var['ULogs'],
        $var['UAlias'], $var['UDelete'], $var['UBrand'], $var['UGroupAssign'],
        $var['BrandDomain'], $var['NumQuota']);

        // Insert the message into the EmailDatabase table (header info only)
        $sql->sqldo( "INSERT INTO AdminUsers (
        Username, Password, Company, Fullname, NumUsers, UAdd, UModify, UPurge,
        USearch, UList, UMigrate, ULogs, UAlias, UDelete, UBrand, UGroupAssign, BrandDomain, NumQuota, DateCreate)
        VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, {$sql->NOW} ) ", $data);

        // Custom branding for the users domain
        if ($var['BrandDomain'] && $var['UGroupAssign'])
        {
                $groups[$var['Username']]['allow_MultiAccounts'] = '1';
                $groups[$var['Username']]['Price'] = '1';
                $groups[$var['Username']]['Quota'] = '51200';
                $groups[$var['Username']]['allow_Forward'] = '1';
                $groups[$var['Username']]['Description'] = "{$var['Company']} SubAdmin";
                $groups[$var['Username']]['allow_SMS'] = '1';
                $groups[$var['Username']]['allow_SpamSettings'] = '1';
                $brand[$var['BrandDomain']]['UGroupAssign'] = "1";
        }

        writeconf();

        // Add the user groups
        $this->_addsubadmin_groups($var['Username'], $var['Domains'], $var['UGroup']);

}

function _addsubadmin_groups($username, $domains, $groups)
{
        global $sql;

    if (!is_array($domains))
        settype($domains, 'array');

        // Toggle if we are standalone mode and setting a number of domains with a comma
        if ($domains[0] && strpos($domains[0], ',') !== false)
                $domains = preg_split('/\s?,\s?/', $domains[0]);

    if (!is_array($groups))
        settype($groups, 'array');

        $domdata = $grpdata = array();

        // Add the domains
        $domquery = "INSERT INTO AdminGroup ( Username, Domain ) VALUES";
        if (is_string($domains))
        {
                $domain = $domains;
                $domquery .= ' ( ? , ? ),';
                array_push($domdata, $username, $domain);
        }
        if (is_array($domains))
        {
                foreach ($domains as $dom)
                {
                        $domain = ($dom)? $dom : '';
                        $domquery .= ' ( ? , ? ),';
                        array_push($domdata, $username, $domain);
                }
        }

        $domquery = preg_replace('/,$/', '', $domquery);

        $res = $sql->sqldo($domquery, $domdata);

        // Add the group
        $grpquery = "INSERT INTO AdminGroup ( Username, UGroup ) VALUES";
        if(is_array($groups))
        foreach ($groups as $grp)
        {
                $group = ($grp)? $grp : '';
                $grpquery .= ' ( ? , ? ),';
                array_push($grpdata, $username, $group );
        }

        $grpquery = preg_replace('/,$/', '', $grpquery);
        $sql->sqldo($grpquery, $grpdata);
}

}

?>
