<?php

################
## Handle email aliases for @Mail - Ben Duncan <ben@calacode.com>
################

require_once('header.php');

require_once('Config.php');

class Alias
{


	################
	## Delete a new mail alias
	## Returns 0 on fail, 1 on success
	################

	function delalias($aliasname, $id)
	{
		global $sql;

		// First, check if the alias already exists in the database
		if (!$sql->getvalue("select count(*) from MailAliases where AliasName = ? and id = ?", array($aliasname, $id)))
			return 0;

	    $sql->sqldo("delete from MailAliases where AliasName = ? and id = ?", array($aliasname, $id));

		return 1;
	}


	################
	## Retrieve a mail alias
	## Returns an array containing a list of mail aliases
	## Returns 0 on fail, 1 on success
	################

	function getaliases($address)
	{
		global $sql;

	    // Find the users domain-name
	    preg_match('/@(.*)/', $address, $m);

	    $domain = $m[1];

	    // SQL escape the email address
	    $domain = $sql->quote($domain);
	    $dbaddress = $sql->quote($address);

		$user = array();

		// PATCH - Edit the order of the Aliases, local aliases first, local accounts, then the global catch-all rule last.

		// Search the local aliases database first
		$user = $sql->sqlarray("select AliasTo from MailAliases where AliasName = $dbaddress");

		// If there are no aliases, check the local users table for a match
		if (!$user[0])
			$user = $sql->sqlarray("select Account from Users where Account = $dbaddress");

		// If no aliases match, try the catch-all
		if (!$user[0])
			$user = $sql->sqlarray("select AliasTo from MailAliases where AliasName = $domain");

		// If no aliases are defined, append the original address to the first element of the array
		if (!$user[0])
			array_push($user, $address);

		return $user;
	}


	################
	## Create a new mail alias
	## Returns 0 on fail, 1 on success, 2 domain does not exist on system
	################

	function makealiases($aliasto, $aliasname)
	{
		#global $sql;
		require_once('SQL.php');
		$sql = new SQL();

		// Find the users domain-name
		if (preg_match('/@(.*)/', $aliasname, $m))
			$domain = $m[1];
		else
			$domain = $aliasname;

		// Take away any whitespace from the name
		list($aliasname, $aliasto) = preg_replace('/\s/', '', array($aliasname, $aliasto));

		$qaliasname = $sql->quote($aliasname);
		$qaliasto = $sql->quote($aliasto);

		// First, check if the alias already exists in the database
		if ($sql->getvalue("select AliasTo from MailAliases where AliasTo = $qaliasto and AliasName = $qaliasname ") )
			return 0;

	    $sql->sqldo("INSERT INTO MailAliases (AliasTo, AliasName, DateCreate) VALUES( $qaliasto, $qaliasname, NOW() )");

		return 1;
	}

	################
	## View an email alias
	## Returns 0 on fail, 1 on success
	################

	function viewaliases($sort, $id=null, $filter=null)
	{
		global $sql;

		if (!$sort)
			$sort = 'id';
		else
			$sort = Filter::cleanSqlFieldNames($sort);

		$user = $db = array();

		if (!$id)
		{
			if ($filter)
			{
				$filter = $sql->quote($filter);
				// Search by a filter
				$q = "select * from MailAliases where AliasName like '%@$filter%'" .
					"or AliasTo like '@$filter%' or AliasName like '$filter%' order by $sort";
				$search = $sql->sqlmultihash($q);
				foreach ($search as $value) {
					array_push($user,$value['id']);
				}
			}
			else
			{
				// Otherwise search all
				$q = "select * from MailAliases order by $sort";
				$search = $sql->sqlmultihash($q);
				foreach ($search as $value) {
					array_push($user,$value['id']);
				}
			}

			return $user;
		}


	    $db = $sql->sqlhash("select * from MailAliases where id=?", $id);
	    return $db;

	}

}
?>
