package Atmail::Log;

use vars qw(@ISA @EXPORT);
use strict;

use Atmail::SQL;

sub new {
    my ($this) = shift;
    my %arg = @_;

    my $class = ref($this) || $this;
    my $self  = {};

    foreach ( keys %arg ) {
        $self->{$_} = $arg{$_};
    }

	$self->{sql} = new Atmail::SQL;
    $self->{Type} = "sql";

    bless( $self, $class );

    return $self;
}

sub DESTROY {
	my($self) = @_;

	$self->{sql}->disconnect();

}

# The generic log function, which calls the appropriate
# sql or flat function
sub log {
    my ( $self, $type, $msg ) = @_;

    my $func = "log_" . $self->{Type};
    return $self->$func( $type, $msg );

}

sub logcheck {
    my ( $self, $type, $ip ) = @_;

    my $func = "logcheck_" . $self->{Type};
    return $self->$func( $type, $ip );

}

# Insert the log into the database
sub log_sql {
    my ( $self, $type, $msg ) = @_;

	$self->{sql}->sqldo("INSERT INTO Log_$type (Account, LogMsg, LogDate) VALUES(?, ?, NOW())", $self->{Account} || 'System', $msg);

    return;
}

# Insert the log into the database
sub log_raw {
    my ( $self, $type, $msg ) = @_;

	$self->{sql}->sqlraw("INSERT INTO Log_$type (Account, LogMsg, LogDate) VALUES(?, ?, NOW())", $self->{Account} || 'System', $msg);

    return;
}

sub logcheck_sql {
    my ( $self, $type, $ip) = @_;

    my $cnt;
	
	$ip = '%' . $ip . '%' if($ip);

    if ($ip && $self->{Account} && $ip !~ /New Account/i) {
        $cnt = $self->{sql}->sqlgetfield( "select count(id) from Log_$type where LogMsg like ? and LogDate > DATE_SUB(NOW(), INTERVAL 24 HOUR) and Account=?", $ip, $self->{Account});
   } else	{
        $cnt = $self->{sql}->sqlgetfield( "select count(id) from Log_$type where LogMsg like ? and LogDate > DATE_SUB(NOW(), INTERVAL 24 HOUR)",  $ip );
 	}

    return $cnt;
}

sub addrelay	{
	my($self, $ip) = @_;

	return 1 if($ip !~ /\d+\.\d+\.\d+\.\d+/);

	# Delete any entries older the 1 hour from the "user" table
	$self->{sql}->sqldo("delete from MailRelay where Account='User' and DateAdded < DATE_SUB(NOW(), INTERVAL " . $self->{sql}->{smtp_popimaprelay_timeout} . " MINUTE)");

	# Add the new IP into the database, only if we do not exist
	if(!$self->{sql}->sqlgetfield("select IPaddress from MailRelay where IPaddress=?", $ip))	{
	$self->{sql}->sqldo("INSERT INTO MailRelay (IPaddress, Account, DateAdded) VALUES(?, 'User', NOW() )", $ip);
	} else	{
	$self->{sql}->sqldo("UPDATE MailRelay set DateAdded=NOW() where IPaddress=? and Account='User'", $ip);	
	}

	return;
}

sub updatelastlogin	{
	my($self, $user) = @_;

	# Return if no user supplied or account invalid
	return if(!$user || $user !~ /@/);

	my $time = time();

	# Update the users last login date
	$self->{sql}->sqldo("update UserSession set LastLogin=? where Account=?", $time, $user);

	return 1;

}

1;