<?php

// +----------------------------------------------------------------+
// | PGP.php														|
// +----------------------------------------------------------------+
// | GnuPG <-> php library											|
// +----------------------------------------------------------------+
// | Source-code (C) CalaCode.com 1998-2006 . All rights reserved	|
// | See http://calacode.com/license.ehtml for license agreement	|
// +----------------------------------------------------------------+
// | Date: January 2006												|
// +----------------------------------------------------------------+

require_once('header.php');

require_once('SQL.php');
require_once('Config.php');

class PGP {

	function PGP($args=null)
	{
		global $pref;

	    $this->wrkDir = $args['wrkDir'];
	    $this->ownFile = $args['ownFile'];
		$this->gpgOpt = "--homedir $this->wrkDir --yes --quiet --no-verbose --no-secmem-warning --no-permission-warning --batch";

		$this->tmp1 = "$this->wrkDir/tmp1.msg";
	    $this->tmp2 = "$this->wrkDir/tmp2.msg";
		$this->ErrorMsg = '';
		$this->Word = '';

		// Create working dir if needed
		if (!is_dir($this->wrkDir))
			if (!$this->makeWrkDir()) return;

		if (!preg_match('/algorithms/i', `{$pref['gpg_path']} --version`)) // gpg present in the command line
		{
			$this->ErrorMsg = "gpg command not present in PATH<br><br>";
			return;
		}

		// Read data from mySQL and save into the temporary PGP path
		preg_match('/.*\/(.*?\@.*?)\//', $this->wrkDir, $m);
		$this->Account = $m[1];

		$this->sql = new SQL();
		$this->sql->table_names($this->Account);

		$gpg_items = array('pubring', 'secring', 'public');

		foreach ($gpg_items as $item)
		{
			if (!$fh = fopen("$this->wrkDir/$item.gpg", 'w'))
				die("Cannot write $this->wrkDir/$item.gpg\n");

			$val = $this->sql->getvalue("select $item from {$this->sql->UserPgp} where Account=?", $this->Account);
			$bytes = strlen($val);
			$bytes_written = fwrite($fh, $val);
			if ($bytes != $bytes_written)
				die("Could not completely write $this->wrkDir/$item.gpg, wrote $bytes_written of $bytes bytes. Ensure filesystem is not full.\n");
			fclose($fh);
		}

		if ($this->checkFile($this->ownFile))
			$this->readWord();
	}


	// generate PGP key pair (and trustdb)
	function genKey($user, $email, $pass, $length='')
	{
		global $pref, $atmail;

		preg_match('/\@(.+)/', $email, $m);
		$domain = $m[1];

		// Specfiy the users realname as their email-address if not specified
		if (!$user)
			$user = $email;

		// Default to 1024 bit encryption
		if (strlen($length) == 0)
			$length = 1024;

	    //generate key pair

		$atmail->rmtree($this->wrkDir);
		mkdir($this->wrkDir, 0777);

		/*$descriptorspec = array(
		   0 => array("pipe", "r"),  // stdin is a pipe that the child will read from
		   1 => array("pipe", "w"),  // stdout is a pipe that the child will write to
		   2 => array("file", "/tmp/pgp-error", 'w')   // stderr is a file
		);
		*/

		//$gpg_proc = proc_open("{$pref['gpg_path']} --gen-key $this->gpgOpt", $descriptorspec, $pipes);


		$input = <<<END
Key-Type: DSA
Key-Length: $length
Subkey-Type: ELG-E
Subkey-Length: $length
Name-Real: $user
Name-Comment: $domain
Name-Email: $email
Passphrase: $pass
Expire-Date: 0
%commit


END;


			// fwrite($pipes[0], $input);

		   	// Start the entropy engine ( using the hdd IRQ/Interupt )
			// The PGP key can take awhile to generate ...

//			while(fgets($pipes[2]))
//				mt_rand(0,10000);

			// It is important that you close any pipes before calling
		   	// proc_close in order to avoid a deadlock
			//fclose($pipes[0]);
			//fclose($pipes[1]);
			//proc_close($gpg_handle);

		// Create tmp file with args for gpg
		if (!file_put_contents("$this->wrkDir/gpg_args", $input))
		{
			$this->ErrorMsg = 'Could not create tmp file for gpg args';
			return;
		}
		/*
		else
		{
			$this->ErrorMsg = "{$pref['gpg_path']} --gen-key command not valid<br><br>";
			return;
		}
		*/

		// Create keys
		system("{$pref['gpg_path']} --gen-key $this->gpgOpt $this->wrkDir/gpg_args");

		// Generate some entropy for gpg
		for ($i=0; $i < 1000; $i++)
			mt_rand(0,10000);

		// Remove tmp arg file
		unlink($this->wrkDir . '/gpg_args');

		// Double check the PGP files exist
		if (!$this->checkFile("$this->wrkDir/pubring.gpg") || !$this->checkFile("$this->wrkDir/secring.gpg") || !$this->checkFile("$this->wrkDir/trustdb.gpg"))
	    {
			$this->ErrorMsg = "Unknown error during key pair generation<br><br>";
			return;
	    }

    	// Generate ASCII public key
		$this->genAsciiPub($email);

		// Fix eventual DB timeout
		if (!$this->sql->ping()) {
			unset($this->sql);
			$this->sql = new SQL();
		}
		// Save the data into a format we can read

		// Delete any previous PGP keys
		$this->sql->sqldo("delete from " . $this->sql->UserPgp . " where Account = ?",  $this->Account);
    	$this->sql->sqldo("INSERT INTO " . $this->sql->UserPgp . " (Account) VALUES (?)", $this->Account);

		// Read each file from the PGP user directory
		foreach (array("trustdb","pubring","secring","public") as $name)
		{
			$content = file_get_contents("$this->wrkDir/$name.gpg");

			// Write the keys to database
	    	$this->sql->sqldo("UPDATE " . $this->sql->UserPgp . " SET $name = ? WHERE Account = ?", array($content, $this->Account));
	    }
	}



	/**
	 * Check that a file exists and is > 0 bytes in length
	 *
	 * @param string $file The full path of the file to check
	 */
	function checkFile($file)
	{
		return (file_exists($file) && filesize($file) > 0);
	}


	// encryption & signing function
	function encrypt($from, $tos, $msg, $encType)
	{
		global $pref;

	    $msgEnc = "";
	    $to = "";
        $from = escapeshellarg($from);

	    foreach ($tos as $t)
	    {
	    	if (empty($t))
	    		continue;

	    	$to .= " -r " . escapeshellarg($t);
	    }

	    if (!$tofile = fopen("$this->tmp1", 'w'))
		{
			$this->ErrorMsg = "Error: opening file $this->tmp1 for writing<br><br>";
			return;
		}

	    fwrite($tofile, $msg);
	    fclose($tofile);

	    //the encryption itself:
	    if (preg_match('/es|se/i', $encType))
	    	$encStr = "--encrypt --sign --passphrase-fd 0";
	    elseif ($encType == 'e' || $encType == 'E')
	    	$encStr = "--encrypt";
	    elseif ($encType == 's' || $encType == 'S')
	  		$encStr = "--clearsign --passphrase-fd 0";

	    $togpg = popen("{$pref['gpg_path']} $this->gpgOpt --default-key $from --always-trust --output $this->tmp2 --armor $encStr $to $this->tmp1", 'w');

	    if ($encType == 's' || $encType == 'S')
			fwrite($togpg, "$this->Word\n");
	    pclose($togpg);

	    if (!file_exists($this->tmp2))
		{
			$this->ErrorMsg = "Error: signing/encrypting failed. Bad password or recipient's public key<br><br>";
			return;
		}

		$msgEnc = file_get_contents($this->tmp2);

	    if ($encType == 's' || $encType == 'S')
			$this->writeWord();

		unlink($this->tmp1);
		unlink($this->tmp2);

	    return $msgEnc;
	}


	// decryption
	function decrypt($encMsg)
	{
		global $pref;

	    $msg = "";

	    $encMsg = $this->formatAsPgp($encMsg);

		if (empty($encMsg))
	    {
	    	$this->ErrorMsg = "Error: No PGP encrypted bloc found in message<br><br>";
	    	return;
	    }

	    if (!$tofile = fopen($this->tmp1, 'w'))
		{
			$this->ErrorMsg = "Error: opening file $this->tmp1 for writing<br><br>";
			return;
		}

	    fwrite($tofile, $encMsg);
	    fclose($tofile);

	    $togpg = popen("{$pref['gpg_path']} $this->gpgOpt --passphrase-fd 0 --output $this->tmp2 --decrypt $this->tmp1", 'w');
	    fwrite($togpg, $this->Word);
	    pclose($togpg);

		// Open the temporary file which contains the encrypted message
	    if (!file_exists($this->tmp2) || !$msg = file_get_contents($this->tmp2))
		{
			$this->ErrorMsg = "Error: Decryption failed (Password or Message Format error)<br><br>";
			return;
		}

		$this->writeWord();
	    unlink($this->tmp1);
		unlink($this->tmp2);

		$msg = nl2br($msg); //simple HTML line-ending fix
		return $msg;
	}


	// verify existence of key in personal keyring
	function existsKey($to)
	{
		global $pref;
        $to = escapeshellarg($to);
	    $gpgMsg = shell_exec("{$pref['gpg_path']} $this->gpgOpt --list-public-keys $to");
	    return (preg_match('/^\s*$/', $gpgMsg))?0:1;
	}


	// import public key in keyring
	function importKey($newKey)
	{
		global $pref;

	    if (!$tofile = fopen($this->tmp1, 'w'))
		{
			$this->ErrorMsg = "importKey: opening file $this->tmp1 for writing.<br><br>";
			return;
		}

	    fwrite($tofile, $newKey);
	    fclose($tofile);

	    $result = shell_exec("{$pref['gpg_path']} $this->gpgOpt --import $this->tmp1");
		if (strpos($result, 'Invalid option "--import"') !== false)
		{
			$this->ErrorMsg = "gpg --import command not valid<br><br>";
			return;
		}

	    return 1;
	}

	//delete public key from personal keyring
	function deleteKey($wrkDir, $email)
	{
		global $pref;
        $wrkDir = escapeshellarg($wrkDir);
        $email = escapeshellarg($email);

	    $this->gpgOpt = "--homedir $wrkDir --yes --quiet --no-verbose --no-secmem-warning --no-permission-warning --batch";

	    $result = shell_exec("{$pref['gpg_path']} $this->gpgOpt --delete-key $email");
		if (strpos($result, 'Invalid option "--delete-key"') !== false)
		{
			$this->ErrorMsg = "gpg --delete-key command not valid<br><br>";
			return;
		}

	    return 1;
	}

	// generate ASCII armored public key
	function genAsciiPub($email)
	{
		global $pref;
        $email = escapeshellarg($email);

	    $result = shell_exec("{$pref['gpg_path']} $this->gpgOpt --output $this->wrkDir/public.gpg --armor --export $email");
		if (strpos($result, 'Invalid option "--export"') !== false)
		{
			$this->ErrorMsg = "gpg --export command not valid<br><br>";
			return;
		}

	    if (!filesize("$this->wrkDir/public.gpg"))
	    {
			$this->ErrorMsg = "Unknown error during ASCII key generation<br><br>";
			return;
	    }
	}

	// retrieve ASCII armored public key
	function retrieveAsciiPub()
	{
		if (!$msg = file_get_contents("$this->wrkDir/public.gpg"))
		{
			$this->ErrorMsg = "Error: Public ASCII key cannot be open for reading<br><br>";
			return;
		}

	    return "\n\n$msg";
	}

	// prepare a well formed message
	function formatAsPgp($msg)
	{
		$msg = preg_replace('/<br>/i', "\n", $msg); //no <br> ends

		$lines = explode("\n", $msg);
		$inPgp = 0;
		$niceMsg = "";

		foreach ($lines as $line)
		{
			$line = trim($line);
			if (empty($line))
				continue; // no empty lines
			if (preg_match('/BEGIN PGP MESSAGE/i', $line))
			{
				$inPgp = 1;

				// Make sure there is a line break before -----BEGIN PGP MESSAGE-----
				$line = str_replace('-----BEGIN PGP MESSAGE-----', "\n-----BEGIN PGP MESSAGE-----", $line);
			}

			if ($inPgp == 0)
				continue; // not in intreresting area yet

			$niceMsg .= "$line\n";
			if (preg_match('/^version/i', $line))
				$niceMsg .= "\n";
			if (preg_match('/END PGP MESSAGE$/i', $line))
				$inPgp = 0;
		}

		return $niceMsg;
	}

	function writeWord()
	{
		$fh = fopen($this->ownFile, 'w');
	    fwrite($fh, $this->Word);
	    fclose($fh);
	}

	function readWord()
	{
	    $this->Word = file_get_contents($this->ownFile);
	}

	// Load the users Public PGP key
	function readPublickey()
	{
		$msg = $this->sql->getvalue("select public from {$this->sql->UserPgp} where Account = ?", $this->Account);
		return $msg;
	}

	function is_error()
	{
		return (empty($this->ErrorMsg)) ? false : true;
	}


	function get_error()
	{
		return $this->ErrorMsg;
	}


	function makeWrkDir()
	{
		$parts = explode('/', $this->wrkDir);
		$path = '/';

		foreach ($parts as $dir)
		{
			$path .= "/$dir";
			if (is_dir($path))
				continue;

			if (!mkdir($path, 0700))
			{
				$this->ErrorMsg = "Cannot create working directory (permission problems?)<br><br>";
				return -1;
			}
		}
	}

}

?>